@@ -12,15 +12,12 @@ from pytype.pytd import mro
 from pytype.pytd import pytd
 
 log = logging.getLogger(__name__)
+_isinstance = abstract_utils._isinstance  # pylint: disable=protected-access
 
 
 # Classes have a metadata dictionary that can store arbitrary metadata for
 # various overlays. We define the dictionary keys here so that they can be
 # shared by abstract.py and the overlays.
-# TODO(mdemello): We choose the key based on the attribute used by the actual
-# decorator for a similar purpose, but we never actually read that attribute. We
-# should just use the decorator name as a key and eliminate one level of
-# indirection.
 _METADATA_KEYS = {
     "dataclasses.dataclass": "__dataclass_fields__",
     # attr.s gets resolved to attr._make.attrs in pyi files but intercepted by
@@ -108,7 +105,13 @@ class Attribute:
 class Class(metaclass=mixin.MixinMeta):  # pylint: disable=undefined-variable
   """Mix-in to mark all class-like values."""
 
-  overloads = ("get_special_attribute", "get_own_new", "call", "compute_mro")
+  overloads = (
+      "_get_class",
+      "call",
+      "compute_mro",
+      "get_own_new",
+      "get_special_attribute",
+  )
 
   def __new__(cls, *unused_args, **unused_kwds):
     """Prevent direct instantiation."""
@@ -120,8 +123,6 @@ class Class(metaclass=mixin.MixinMeta):  # pylint: disable=undefined-variable
     if metaclass is None:
       metaclass = self._get_inherited_metaclass()
     if metaclass:
-      # TODO(rechen): Check that the metaclass is a (non-strict) subclass of the
-      # metaclasses of the base classes.
       self.cls = metaclass
     # Key-value store of metadata for overlays to use.
     self.metadata = {}
@@ -136,6 +137,9 @@ class Class(metaclass=mixin.MixinMeta):  # pylint: disable=undefined-variable
     if self.is_test_class():
       self.additional_init_methods.append("setUp")
 
+  def _get_class(self):
+    return self.ctx.convert.type_type
+
   def bases(self):
     return []
 
@@ -163,29 +167,30 @@ class Class(metaclass=mixin.MixinMeta):  # pylint: disable=undefined-variable
     """Get the attributes defined by this class."""
     raise NotImplementedError(self.__class__.__name__)
 
-  def has_protocol_parent(self):
+  def has_protocol_base(self):
     """Whether this class inherits directly from typing.Protocol."""
-    if self.isinstance_PyTDClass():
-      for parent in self.pytd_cls.parents:
-        if parent.name == "typing.Protocol":
+    if _isinstance(self, "PyTDClass"):
+      for base in self.pytd_cls.bases:
+        if base.name == "typing.Protocol":
           return True
-    elif self.isinstance_InterpreterClass():
-      for parent_var in self._bases:
-        for parent in parent_var.data:
-          if (parent.isinstance_PyTDClass() and
-              parent.full_name == "typing.Protocol"):
+    elif _isinstance(self, "InterpreterClass"):
+      for base_var in self._bases:
+        for base in base_var.data:
+          if (_isinstance(base, "PyTDClass") and
+              base.full_name == "typing.Protocol"):
             return True
     return False
 
   def _init_protocol_attributes(self):
     """Compute this class's protocol attributes."""
-    if self.isinstance_ParameterizedClass():
+    if _isinstance(self, "ParameterizedClass"):
       self.protocol_attributes = self.base_cls.protocol_attributes
       return
-    if not self.has_protocol_parent():
+    if not self.has_protocol_base():
       self.protocol_attributes = set()
       return
-    if self.isinstance_PyTDClass() and self.pytd_cls.name.startswith("typing."):
+    if (_isinstance(self, "PyTDClass") and
+        self.pytd_cls.name.startswith("typing.")):
       protocol_attributes = set()
       if self.pytd_cls.name == "typing.Mapping":
         # Append Mapping-specific attributes to forbid matching against classes
@@ -219,7 +224,7 @@ class Class(metaclass=mixin.MixinMeta):  # pylint: disable=undefined-variable
     """Compute and cache whether the class sets its own boolean value."""
     # A class's instances can evaluate to False if it defines __bool__ or
     # __len__.
-    if self.isinstance_ParameterizedClass():
+    if _isinstance(self, "ParameterizedClass"):
       self.overrides_bool = self.base_cls.overrides_bool
       return
     for cls in self.mro:
@@ -252,20 +257,20 @@ class Class(metaclass=mixin.MixinMeta):  # pylint: disable=undefined-variable
     self.abstract_methods = abstract_methods
 
   def _has_explicit_abcmeta(self):
-    return any(parent.full_name == "abc.ABCMeta" for parent in self.cls.mro)
+    return any(base.full_name == "abc.ABCMeta" for base in self.cls.mro)
 
   def _has_implicit_abcmeta(self):
     """Whether the class should be considered implicitly abstract."""
     # Protocols must be marked as abstract to get around the
     # [ignored-abstractmethod] check for interpreter classes.
-    if not self.isinstance_InterpreterClass():
+    if not _isinstance(self, "InterpreterClass"):
       return False
-    # We check self._bases (immediate parents) instead of self.mro because our
+    # We check self._bases (immediate bases) instead of self.mro because our
     # builtins and typing stubs are inconsistent about implementing abstract
     # methods, and we don't want [not-instantiable] errors all over the place
     # because a class has Protocol buried in its MRO.
     for var in self._bases:
-      if any(parent.full_name == "typing.Protocol" for parent in var.data):
+      if any(base.full_name == "typing.Protocol" for base in var.data):
         return True
     return False
 
@@ -301,12 +306,10 @@ class Class(metaclass=mixin.MixinMeta):  # pylint: disable=undefined-variable
     node, init = self.ctx.attribute_handler.get_attribute(
         node, self.cls, "__init__")
     if not init or not any(
-        f.isinstance_SignedFunction() for f in init.data):
+        _isinstance(f, "SignedFunction") for f in init.data):
       # Only SignedFunctions (InterpreterFunction and SimpleFunction) have
       # interesting side effects.
       return node
-    # TODO(rechen): The signature is (cls, name, bases, dict); should we fill in
-    # the last arg more precisely?
     args = function.Args(
         posargs=(self.to_variable(node),
                  self.ctx.convert.build_string(node, self.name),
@@ -340,7 +343,7 @@ class Class(metaclass=mixin.MixinMeta):  # pylint: disable=undefined-variable
       return node, None
     if len(new.bindings) == 1:
       f = new.bindings[0].data
-      if (f.isinstance_AMBIGUOUS_OR_EMPTY() or
+      if (_isinstance(f, "AMBIGUOUS_OR_EMPTY") or
           self.ctx.convert.object_type.is_object_new(f)):
         # Instead of calling object.__new__, our abstract classes directly
         # create instances of themselves.
@@ -389,12 +392,29 @@ class Class(metaclass=mixin.MixinMeta):  # pylint: disable=undefined-variable
       key = node
     assert key
     if key not in self._instance_cache:
-      self._instance_cache[key] = self._to_instance(container)
+      self._instance_cache[key] = self.ctx.abstract_classes_for_submodules[
+          "Instance"].from_value(self, container)
     return self._instance_cache[key]
 
+  def _check_not_instantiable(self):
+    """Report [not-instantiable] if the class cannot be instantiated."""
+    # We report a not-instantiable error if all of the following are true:
+    # - The class is abstract.
+    # - It was not created from an explicit type annotation.
+    # - The instantiation is not occuring inside one of the class's own methods.
+    # We check the last condition by seeing whether ctx.vm.frame.func is an
+    # InterpreterFunction whose name starts with "<class>."
+    if not self.is_abstract or self.from_annotation:
+      return
+    if self.ctx.vm.frame and self.ctx.vm.frame.func:
+      calling_func = self.ctx.vm.frame.func.data
+      if (_isinstance(calling_func, "InterpreterFunction") and
+          calling_func.name.startswith(f"{self.name}.")):
+        return
+    self.ctx.errorlog.not_instantiable(self.ctx.vm.frames, self)
+
   def call(self, node, func, args):
-    if self.is_abstract and not self.from_annotation:
-      self.ctx.errorlog.not_instantiable(self.ctx.vm.frames, self)
+    self._check_not_instantiable()
     node, variable = self._call_new_and_init(node, func, args)
     if variable is None:
       value = self._new_instance(None, node, args)
@@ -417,7 +437,8 @@ class Class(metaclass=mixin.MixinMeta):  # pylint: disable=undefined-variable
       # Treat this class as a parameterized container in an annotation. We do
       # not need to worry about the class not being a container: in that case,
       # AnnotationContainer's param length check reports an appropriate error.
-      container = self.to_annotation_container()
+      container = self.ctx.abstract_classes_for_submodules[
+          "AnnotationContainer"].from_value(self)
       return container.get_special_attribute(node, name, valself)
     return Class.super(self.get_special_attribute)(node, name, valself)
 
@@ -439,7 +460,7 @@ class Class(metaclass=mixin.MixinMeta):  # pylint: disable=undefined-variable
     for row in bases:
       baselist = []
       for base in row:
-        if base.isinstance_ParameterizedClass():
+        if _isinstance(base, "ParameterizedClass"):
           base2cls[base.base_cls] = base
           baselist.append(base.base_cls)
         else:
