@@ -13,8 +13,9 @@ import sys
 from pytype import datatypes
 from pytype import errors
 from pytype import imports_map_loader
-from pytype import load_pytd
+from pytype import module_utils
 from pytype import utils
+from pytype.pytd import pytd_utils
 from pytype.typegraph import cfg_utils
 
 
@@ -153,9 +154,8 @@ FEATURE_FLAGS = [
     ("--strict_namedtuple_checks", False,
      ("Enable stricter namedtuple checks, such as unpacking and "
       "'typing.Tuple' compatibility.")),
-    ("--gen-stub-imports", False,
-     ("Generate import statements (`import x`) rather than constants "
-      "(`x: module`) for module names in stub files.")),
+    ("--fix-module-collisions", True,
+     ("Fix collisions between module and local names in pyi files.")),
 ]
 
 
@@ -336,6 +336,10 @@ def add_debug_options(o):
       "-S", "--timestamp-logs", action="store_true",
       dest="timestamp_logs", default=None,
       help=("Add timestamps to the logs"))
+  o.add_argument(
+      "--debug-logs", action="store_true",
+      dest="debug_logs", default=None,
+      help=("Add debugging information to the logs"))
   o.add_argument(
       "--verify-pickle", action="store_true", default=False,
       dest="verify_pickle",
@@ -441,8 +445,8 @@ class Postprocessor:
     if pickle_output:
       if self.output_options.output is None:
         self.error("Can't use without --output", "pickle-output")
-      elif not load_pytd.is_pickle(self.output_options.output):
-        self.error("Must specify %s file for --output" % load_pytd.PICKLE_EXT,
+      elif not pytd_utils.IsPickle(self.output_options.output):
+        self.error("Must specify %s file for --output" % pytd_utils.PICKLE_EXT,
                    "pickle-output")
     self.output_options.pickle_output = pickle_output
 
@@ -454,7 +458,7 @@ class Postprocessor:
       self.error("Can't use without --pickle-output", "verify-pickle")
     else:
       self.output_options.verify_pickle = self.output_options.output.replace(
-          load_pytd.PICKLE_EXT, ".pyi")
+          pytd_utils.PICKLE_EXT, ".pyi")
 
   @uses(["input", "show_config", "pythonpath", "version"])
   def _store_generate_builtins(self, generate_builtins):
@@ -482,7 +486,7 @@ class Postprocessor:
     else:
       self.output_options.typeshed = True
 
-  @uses(["timestamp_logs"])
+  @uses(["timestamp_logs", "debug_logs"])
   def _store_verbosity(self, verbosity):
     """Configure logging."""
     if not -1 <= verbosity < len(LOG_LEVELS):
@@ -498,20 +502,17 @@ class Postprocessor:
     """Configure the python version."""
     if python_version:
       if isinstance(python_version, str):
-        self.output_options.python_version = utils.version_from_string(
-            python_version)
-      elif isinstance(python_version, int):
-        self.output_options.python_version = utils.full_version_from_major(
-            python_version)
+        version = utils.version_from_string(python_version)
       else:
-        self.output_options.python_version = python_version
+        version = python_version
     else:
-      self.output_options.python_version = sys.version_info[:2]
-    if len(self.output_options.python_version) != 2:
+      version = sys.version_info[:2]
+    if len(version) != 2:
       self.error(
           "--python_version must be <major>.<minor>: %r" % python_version)
     # Check that we have a version supported by pytype.
-    utils.validate_version(self.output_options.python_version)
+    utils.validate_version(version)
+    self.output_options.python_version = version
 
     if utils.can_compile_bytecode_natively(self.output_options.python_version):
       # pytype does not need an exe for bytecode compilation. Abort early to
@@ -574,7 +575,7 @@ class Postprocessor:
   @uses(["input", "pythonpath"])
   def _store_module_name(self, module_name):
     if module_name is None:
-      module_name = load_pytd.get_module_name(
+      module_name = module_utils.get_module_name(
           self.output_options.input, self.output_options.pythonpath)
     self.output_options.module_name = module_name
 
@@ -601,7 +602,7 @@ class Postprocessor:
                  "separated by %r" % (item, os.pathsep))
 
 
-def _set_verbosity(verbosity, timestamp_logs):
+def _set_verbosity(verbosity, timestamp_logs, debug_logs):
   """Set the logging verbosity."""
   if verbosity >= 0:
     basic_logging_level = LOG_LEVELS[verbosity]
@@ -613,9 +614,13 @@ def _set_verbosity(verbosity, timestamp_logs):
     # When calling pytype as a library, override the caller's logging level.
     logging.getLogger().setLevel(basic_logging_level)
   else:
-    fmt = "%(levelname)s:%(name)s %(message)s"
-    if timestamp_logs:
-      fmt = "%(relativeCreated)f " + fmt
+    if debug_logs:
+      fmt = (":%(relativeCreated)f:%(levelname)s:%(name)s:%(funcName)s:"
+             "%(lineno)s: %(message)s")
+    else:
+      fmt = "%(levelname)s:%(name)s %(message)s"
+      if timestamp_logs:
+        fmt = "%(relativeCreated)f " + fmt
     logging.basicConfig(level=basic_logging_level, format=fmt)
 
 
@@ -635,7 +640,7 @@ def verbosity_from(options):
     Nothing.
   """
   level = logging.getLogger().getEffectiveLevel()
-  _set_verbosity(options.verbosity, options.timestamp_logs)
+  _set_verbosity(options.verbosity, options.timestamp_logs, options.debug_logs)
   try:
     yield
   finally:
