@@ -237,7 +237,6 @@ class Converter(utils.ContextWeakrefMixin):
 
   def build_list(self, node, content):
     """Create a VM list from the given sequence."""
-    # TODO(rechen): set T to empty if there is nothing in content
     content = [var.AssignToNewVariable(node) for var in content]
     return abstract.List(content, self.ctx).to_variable(node)
 
@@ -757,7 +756,7 @@ class Converter(utils.ContextWeakrefMixin):
           content = tuple(self.constant_to_var(abstract_utils.AsInstance(p),
                                                subst, get_node())
                           for p in cls.parameters)
-          return abstract.Tuple(content, self.ctx)
+          return self.tuple_to_value(content)
         elif isinstance(cls, pytd.CallableType):
           clsval = self.constant_to_value(cls, subst, self.ctx.root_node)
           return abstract.Instance(clsval, self.ctx)
