@@ -3,12 +3,12 @@
 import bisect
 import collections
 import io
-import itertools
 import keyword
 import logging
 import re
 import sys
 import tokenize
+from typing import Collection
 
 from pytype import blocks
 from pytype import utils
@@ -22,6 +22,14 @@ _DOCSTRING_RE = re.compile(r"^\s*(\"\"\"|''')")
 _DECORATOR_RE = re.compile(r"^\s*@(\w+)([(]|\s*$)")
 _ALL_ERRORS = "*"  # Wildcard for disabling all errors.
 
+_FUNCTION_CALL_ERRORS = (
+    "duplicate-keyword",
+    "missing-parameter",
+    "wrong-arg-count",
+    "wrong-arg-types",
+    "wrong-keyword-args",
+)
+
 
 class _DirectiveError(Exception):
   pass
@@ -226,6 +234,107 @@ def _collect_bytecode(ordered_code):
   return bytecode_blocks
 
 
+def _adjust_line_number(line, allowed_lines, min_line=1):
+  adjusted_line = line
+  while adjusted_line not in allowed_lines and adjusted_line >= min_line:
+    adjusted_line -= 1
+  return adjusted_line if adjusted_line >= min_line else None
+
+
+def _is_function_call(opcode_name):
+  return opcode_name.startswith("CALL_") or opcode_name in {
+      "BINARY_SUBSCR",
+      "COMPARE_OP",
+      "FOR_ITER",
+  }
+
+
+def _is_funcdef_op(opcode_name):
+  """Checks whether the opcode may appear in a function signature."""
+  return opcode_name.startswith("LOAD_") or opcode_name in {
+      "BINARY_SUBSCR",
+      "BUILD_TUPLE",
+  }
+
+
+def _is_load_attribute_op(opcode_name):
+  """Checks whether the opcode loads an attribute."""
+  return (opcode_name.startswith("GET_") or
+          opcode_name.startswith("UNPACK_") or
+          opcode_name in {
+              "LOAD_ATTR",
+              "LOAD_METHOD",
+              "SETUP_WITH",
+          })
+
+
+class _OpcodeLines:
+  """Stores opcode line numbers for Director.adjust_line_numbers()."""
+
+  class Call:
+
+    def __init__(self, line):
+      self.line = line
+      self.children = set()
+
+    def __repr__(self):
+      return f"call @ line {self.line} ({len(self.children)} nested calls)"
+
+  def __init__(
+      self,
+      store_lines: Collection[int],
+      make_function_lines: Collection[int],
+      non_funcdef_lines: Collection[int],
+      load_attr_lines: Collection[int],
+      call_lines: Collection[Call]):
+    self.store_lines = store_lines
+    self.make_function_lines = make_function_lines
+    self.non_funcdef_lines = non_funcdef_lines
+    self.load_attr_lines = load_attr_lines
+    # We transform call_lines into a line->Call mapping so that
+    # _adjust_line_number can treat it as a Collection[int] like the other
+    # *_lines attributes.
+    self.call_lines = {call.line: call for call in call_lines}
+
+  @classmethod
+  def from_code(cls, code):
+    """Builds an _OpcodeLines from a code object."""
+    store_lines = set()
+    make_function_lines = set()
+    non_funcdef_lines = set()
+    load_attr_lines = set()
+    all_call_lines = []
+    for block in _collect_bytecode(code):
+      call_lines = []
+      for opcode in block:
+        if opcode.name.startswith("STORE_"):
+          store_lines.add(opcode.line)
+        elif opcode.name == "MAKE_FUNCTION":
+          make_function_lines.add(opcode.line)
+        elif _is_load_attribute_op(opcode.name):
+          load_attr_lines.add(opcode.line)
+        elif _is_function_call(opcode.name):
+          # Function calls can be nested, so we represent them as a sequence of
+          # call trees. As opcodes are typically ordered by increasing line
+          # number, we detect nested calls via decreasing line numbers. For
+          # example, for:
+          #   top_level1()  # line 1
+          #   top_level2(  # line 2
+          #     nested1(  # line 3
+          #       more_nested()),  # line 4
+          #     nested2())  # line 5
+          # the sequence of line numbers is [1, 4, 3, 5, 2].
+          call = _OpcodeLines.Call(opcode.line)
+          while call_lines and opcode.line < call_lines[-1].line:
+            call.children.add(call_lines.pop())
+          call_lines.append(call)
+        if not _is_funcdef_op(opcode.name):
+          non_funcdef_lines.add(opcode.line)
+      all_call_lines.extend(call_lines)
+    return cls(store_lines, make_function_lines, non_funcdef_lines,
+               load_attr_lines, all_call_lines)
+
+
 class Director:
   """Holds all of the directive information for a source file."""
 
@@ -460,29 +569,12 @@ class Director:
             lineno not in self._disables[_ALL_ERRORS] and
             lineno not in self._disables[error.name])
 
-  def adjust_line_numbers(self, code):
-    """Uses the bytecode to adjust line numbers."""
-    store_lines = set()
-    make_function_lines = set()
-    non_load_lines = set()
-    for opcode in itertools.chain.from_iterable(_collect_bytecode(code)):
-      if opcode.name.startswith("STORE_"):
-        store_lines.add(opcode.line)
-      elif opcode.name == "MAKE_FUNCTION":
-        make_function_lines.add(opcode.line)
-      if not opcode.name.startswith("LOAD_"):
-        non_load_lines.add(opcode.line)
-
-    def adjust(line, allowed_lines, min_line=1):
-      adjusted_line = line
-      while adjusted_line not in allowed_lines and adjusted_line >= min_line:
-        adjusted_line -= 1
-      return adjusted_line if adjusted_line >= min_line else None
-
-    # Process type comments.
+  def _adjust_line_numbers_for_type_comments(self, opcode_lines):
+    """Adjusts line numbers for `# type:` comments."""
     for type_comment_set in self._type_comments:
       for line, comment in sorted(type_comment_set.type_comments.items()):
-        adjusted_line = adjust(line, store_lines, type_comment_set.start_line)
+        adjusted_line = _adjust_line_number(
+            line, opcode_lines.store_lines, type_comment_set.start_line)
         if not adjusted_line:
           # vm._FindIgnoredTypeComments will take care of error reporting.
           continue
@@ -493,9 +585,10 @@ class Director:
           type_comment_set.type_comments[adjusted_line] = comment
           del type_comment_set.type_comments[line]
 
-    # Process decorators.
+  def _adjust_line_numbers_for_decorators(self, opcode_lines):
     for line in sorted(self._decorators):
-      adjusted_line = adjust(line, make_function_lines)
+      adjusted_line = _adjust_line_number(
+          line, opcode_lines.make_function_lines)
       if not adjusted_line:
         log.error(
             "No MAKE_FUNCTION opcode found for decorator on line %d", line)
@@ -503,9 +596,9 @@ class Director:
         self._decorators.add(adjusted_line)
         self._decorators.remove(line)
 
-    # Process variable annotations.
+  def _adjust_line_numbers_for_variable_annotations(self, opcode_lines):
     for line, annot in sorted(self._variable_annotations.items()):
-      adjusted_line = adjust(line, store_lines)
+      adjusted_line = _adjust_line_number(line, opcode_lines.store_lines)
       if not adjusted_line:
         log.error(
             "No STORE_* opcode found for annotation %r on line %d", annot, line)
@@ -514,18 +607,66 @@ class Director:
         self._variable_annotations[adjusted_line] = annot
         del self._variable_annotations[line]
 
-    # Process annotation-type-mismatch directives.
-    if "annotation-type-mismatch" in self._disables:
-      lines = self._disables["annotation-type-mismatch"].lines
+  def _adjust_line_numbers_for_error_directives(self, opcode_lines):
+    """Adjusts line numbers for error directives."""
+    for error_class in _FUNCTION_CALL_ERRORS + (
+        "annotation-type-mismatch", "attribute-error"):
+      if error_class not in self._disables:
+        continue
+      lines = self._disables[error_class].lines
       for line, membership in sorted(lines.items()):
-        min_line = line
-        # In Python 3.8+, the MAKE_FUNCTION opcode's line number is the first
-        # line of the function signature, so we need to skip any LOAD_* opcodes
-        # in between.
-        while min_line not in non_load_lines and min_line > 1:
-          min_line -= 1
-        adjusted_line = adjust(
-            line, store_lines | make_function_lines, min_line)
+        if error_class == "annotation-type-mismatch":
+          min_line = line
+          # In Python 3.8+, the MAKE_FUNCTION opcode's line number is the first
+          # line of the function signature, so we need to skip any opcodes
+          # associated with the signature in between.
+          while min_line not in opcode_lines.non_funcdef_lines and min_line > 1:
+            min_line -= 1
+          allowed_lines = (
+              opcode_lines.store_lines | opcode_lines.make_function_lines)
+        elif error_class == "attribute-error":
+          min_line = 1
+          allowed_lines = opcode_lines.load_attr_lines
+        else:
+          min_line = 1
+          allowed_lines = opcode_lines.call_lines
+        adjusted_line = _adjust_line_number(line, allowed_lines, min_line)
         if adjusted_line and adjusted_line != line:
-          lines[adjusted_line] = membership
-          del lines[line]
+          if error_class in ("annotation-type-mismatch", "attribute-error"):
+            lines[adjusted_line] = membership
+            del lines[line]
+          else:
+            # 'adjusted_line' points to the top-level function call that
+            # contains 'line'. We apply the directive on 'line' to the top-level
+            # call as well as all nested calls between 'adjusted_line' and
+            # 'line'. For example, in:
+            # top_level_call(
+            #     nested1(),
+            #     nested2(),  # directive
+            #     nested3())
+            # the disable directive applies to top_level_call, nested1, and
+            # nested 2, but not nested3.
+            stack = [opcode_lines.call_lines[adjusted_line]]
+            found = False
+            while stack:
+              call = stack.pop()
+              if call.line > line:
+                continue
+              if call.line == line:
+                found = True
+              else:
+                lines[call.line] = membership
+              stack.extend(call.children)
+            # If 'line' does not itself contain a function call, then we delete
+            # the directive on 'line' after applying it to all the relevant
+            # lines that do contain function calls.
+            if not found:
+              del lines[line]
+
+  def adjust_line_numbers(self, code):
+    """Uses the bytecode to adjust line numbers."""
+    opcode_lines = _OpcodeLines.from_code(code)
+    self._adjust_line_numbers_for_type_comments(opcode_lines)
+    self._adjust_line_numbers_for_decorators(opcode_lines)
+    self._adjust_line_numbers_for_variable_annotations(opcode_lines)
+    self._adjust_line_numbers_for_error_directives(opcode_lines)
