@@ -403,7 +403,6 @@ class ErrorLogBase:
       csv_file = csv.writer(f, delimiter=",")
       for error in self.unique_sorted_errors():
         # pylint: disable=protected-access
-        # TODO(b/159038861): Add _methodname
         if error._details and error._traceback:
           details = error._details + "\n\n" + error._traceback
         elif error._traceback:
@@ -585,7 +584,7 @@ class ErrorLog(ErrorLogBase):
   def _print_noniterable_str_error(self, error):
     return (
         f"Note: {error.left_type.name} does not match iterables by default. "
-        "Learn more: https://github.com/google/pytype/blob/master/docs/faq.md#why-doesnt-str-match-against-string-iterables")
+        "Learn more: https://github.com/google/pytype/blob/main/docs/faq.md#why-doesnt-str-match-against-string-iterables")
 
   def _prepare_errorlog_details(self, bad):
     protocol_details = set()
@@ -1139,9 +1138,8 @@ class ErrorLog(ErrorLogBase):
       self.error(stack, actual, details=details)
 
   @_error_name("annotation-type-mismatch")
-  def annotation_type_mismatch(self, stack, annot, binding, name, details=None):
-    # TODO(rpalaguachi): Include detailed error message for
-    # NonIterableStringError when a `str` is assigned to a string iterable.
+  def annotation_type_mismatch(
+      self, stack, annot, binding, name, protocol_err, nis_err, details=None):
     """Invalid combination of annotation and assignment."""
     if annot is None:
       return
@@ -1151,6 +1149,10 @@ class ErrorLog(ErrorLogBase):
     if actual_string == "None":
       annot_string += f" (Did you mean 'typing.Optional[{annot_string}]'?)"
     additional_details = f"\n\n{details}" if details else ""
+    if protocol_err:
+      additional_details += "\n" + self._print_protocol_error(protocol_err)
+    if nis_err:
+      additional_details += "\n" + self._print_noniterable_str_error(nis_err)
     details = ("Annotation: %s\n" % annot_string +
                "Assignment: %s" % actual_string +
                additional_details)
@@ -1177,9 +1179,9 @@ class ErrorLog(ErrorLogBase):
       mutations: a dict of {parameter name: (annotated types, new types)}
       name: the variable name (or None)
     """
-    for parent in obj.cls.mro:
-      if isinstance(parent, abstract.ParameterizedClass):
-        cls = parent
+    for base in obj.cls.mro:
+      if isinstance(base, abstract.ParameterizedClass):
+        cls = base
         break
     else:
       assert False, f"{obj.cls.full_name} is not a container"
