@@ -16,7 +16,7 @@ from pytype import load_pytd
 from pytype import utils
 from pytype.pyc import pyc
 from pytype.pyi import parser
-from pytype.pytd import builtins as pytd_builtins
+from pytype.pytd import builtin_stubs as pytd_builtins
 from pytype.pytd import optimize
 from pytype.pytd import pytd_utils
 from pytype.pytd import serialize_ast
@@ -105,13 +105,13 @@ def generate_pyi(src, options=None, loader=None):
     mod.Visit(visitors.VerifyVisitor())
     mod = optimize.Optimize(mod,
                             builtins,
-                            # TODO(b/159038508): Add FLAGs for these
                             lossy=False,
                             use_abcs=False,
                             max_union=7,
                             remove_mutable=False)
     mod = pytd_utils.CanonicalOrdering(mod, sort_signatures=True)
-    result = pytd_utils.Print(mod)
+    result = pytd_utils.Print(
+        mod, fix_module_collisions=options.fix_module_collisions)
     log.info("=========== pyi optimized =============")
     log.info("\n%s", result)
     log.info("========================================")
@@ -136,7 +136,8 @@ def check_or_generate_pyi(options, loader=None):
 
   errorlog = errors.ErrorLog()
   result = pytd_builtins.DEFAULT_SRC
-  ast = pytd_builtins.GetDefaultAst(options.gen_stub_imports)
+  ast = pytd_builtins.GetDefaultAst(
+      parser.PyiOptions.from_toplevel_options(options))
   try:
     src = read_source_file(options.input, options.open_function)
     if options.check:
@@ -234,8 +235,7 @@ def write_pickle(ast, options, loader=None):
   except parser.ParseError as e:
     if options.nofail:
       ast = serialize_ast.PrepareForExport(
-          options.module_name,
-          pytd_builtins.GetDefaultAst(options.gen_stub_imports), loader)
+          options.module_name, loader.get_default_ast(), loader)
       log.warning("***Caught exception: %s", str(e), exc_info=True)
     else:
       raise
