@@ -6,8 +6,10 @@ import io
 import os
 import textwrap
 
+from pytype import config
 from pytype import file_utils
 from pytype import load_pytd
+from pytype import module_utils
 from pytype.pytd import pytd
 from pytype.pytd import pytd_utils
 from pytype.pytd import serialize_ast
@@ -24,7 +26,8 @@ class _LoaderTest(test_base.UnitTest):
     with file_utils.Tempdir() as d:
       for name, contents in kwargs.items():
         d.create_file(f"{name}.pyi", contents)
-      yield load_pytd.Loader(None, self.python_version, pythonpath=[d.path])
+      yield load_pytd.Loader(config.Options.create(
+          python_version=self.python_version, pythonpath=d.path))
 
   def _import(self, **kwargs):
     with self._setup_loader(**kwargs) as loader:
@@ -54,19 +57,21 @@ class ImportPathsTest(_LoaderTest):
         ("../../foo.py", [".."], None)
     ]
     for filename, pythonpath, expected in test_cases:
-      module = load_pytd.get_module_name(filename, pythonpath)
+      module = module_utils.get_module_name(filename, pythonpath)
       self.assertEqual(module, expected)
 
   def test_builtin_sys(self):
-    loader = load_pytd.Loader("base", self.python_version)
+    loader = load_pytd.Loader(config.Options.create(
+        module_name="base", python_version=self.python_version))
     ast = loader.import_name("sys")
     self.assertTrue(ast.Lookup("sys.exit"))
 
   def test_basic(self):
     with file_utils.Tempdir() as d:
       d.create_file("path/to/some/module.pyi", "def foo(x:int) -> str: ...")
-      loader = load_pytd.Loader(
-          "base", self.python_version, pythonpath=[d.path])
+      loader = load_pytd.Loader(config.Options.create(
+          module_name="base", python_version=self.python_version,
+          pythonpath=d.path))
       ast = loader.import_name("path.to.some.module")
       self.assertTrue(ast.Lookup("path.to.some.module.foo"))
 
@@ -75,8 +80,9 @@ class ImportPathsTest(_LoaderTest):
       with file_utils.Tempdir() as d2:
         d1.create_file("dir1/module1.pyi", "def foo1() -> str: ...")
         d2.create_file("dir2/module2.pyi", "def foo2() -> str: ...")
-        loader = load_pytd.Loader(
-            "base", self.python_version, pythonpath=[d1.path, d2.path])
+        loader = load_pytd.Loader(config.Options.create(
+            module_name="base", python_version=self.python_version,
+            pythonpath=f"{d1.path}:{d2.path}"))
         module1 = loader.import_name("dir1.module1")
         module2 = loader.import_name("dir2.module2")
         self.assertTrue(module1.Lookup("dir1.module1.foo1"))
@@ -85,15 +91,17 @@ class ImportPathsTest(_LoaderTest):
   def test_init(self):
     with file_utils.Tempdir() as d1:
       d1.create_file("baz/__init__.pyi", "x = ... # type: int")
-      loader = load_pytd.Loader(
-          "base", self.python_version, pythonpath=[d1.path])
+      loader = load_pytd.Loader(config.Options.create(
+          module_name="base", python_version=self.python_version,
+          pythonpath=d1.path))
       self.assertTrue(loader.import_name("baz").Lookup("baz.x"))
 
   def test_builtins(self):
     with file_utils.Tempdir() as d:
       d.create_file("foo.pyi", "x = ... # type: int")
-      loader = load_pytd.Loader(
-          "base", self.python_version, pythonpath=[d.path])
+      loader = load_pytd.Loader(config.Options.create(
+          module_name="base", python_version=self.python_version,
+          pythonpath=d.path))
       mod = loader.import_name("foo")
       self.assertEqual("builtins.int", mod.Lookup("foo.x").type.cls.name)
       self.assertEqual("builtins.int", mod.Lookup("foo.x").type.name)
@@ -101,20 +109,24 @@ class ImportPathsTest(_LoaderTest):
   def test_no_init(self):
     with file_utils.Tempdir() as d:
       d.create_directory("baz")
-      loader = load_pytd.Loader(
-          "base", self.python_version, pythonpath=[d.path])
+      loader = load_pytd.Loader(config.Options.create(
+          module_name="base", python_version=self.python_version,
+          pythonpath=d.path))
       self.assertTrue(loader.import_name("baz"))
 
   def test_no_init_imports_map(self):
     with file_utils.Tempdir() as d:
       d.create_directory("baz")
       os.chdir(d.path)
-      loader = load_pytd.Loader(
-          "base", self.python_version, imports_map={}, pythonpath=[""])
+      loader = load_pytd.Loader(config.Options.create(
+          module_name="base", python_version=self.python_version,
+          pythonpath=""))
+      loader.options.tweak(imports_map={})
       self.assertFalse(loader.import_name("baz"))
 
   def test_stdlib(self):
-    loader = load_pytd.Loader("base", self.python_version)
+    loader = load_pytd.Loader(config.Options.create(
+        module_name="base", python_version=self.python_version))
     ast = loader.import_name("io")
     self.assertTrue(ast.Lookup("io.StringIO"))
 
@@ -122,8 +134,9 @@ class ImportPathsTest(_LoaderTest):
     with file_utils.Tempdir() as d:
       d.create_file("module1.pyi", "def get_bar() -> module2.Bar: ...")
       d.create_file("module2.pyi", "class Bar:\n  pass")
-      loader = load_pytd.Loader(
-          "base", self.python_version, pythonpath=[d.path])
+      loader = load_pytd.Loader(config.Options.create(
+          module_name="base", python_version=self.python_version,
+          pythonpath=d.path))
       module1 = loader.import_name("module1")
       f, = module1.Lookup("module1.get_bar").signatures
       self.assertEqual("module2.Bar", f.return_type.cls.name)
@@ -140,8 +153,9 @@ class ImportPathsTest(_LoaderTest):
         class Bar:
           pass
       """)
-      loader = load_pytd.Loader(
-          "base", self.python_version, pythonpath=[d.path])
+      loader = load_pytd.Loader(config.Options.create(
+          module_name="base", python_version=self.python_version,
+          pythonpath=d.path))
       foo = loader.import_name("foo")
       bar = loader.import_name("bar")
       f1, = foo.Lookup("foo.get_bar").signatures
@@ -175,8 +189,8 @@ class ImportPathsTest(_LoaderTest):
       AnyPath = PathLike[str]
     """) as loader:
       loader.finish_and_verify_ast(
-          loader.load_file(
-              "target", os.path.join(loader.pythonpath[0], "target.pyi")))
+          loader.load_file("target", os.path.join(
+              loader.options.pythonpath[0], "target.pyi")))
 
   def test_relative(self):
     with file_utils.Tempdir() as d:
@@ -185,9 +199,9 @@ class ImportPathsTest(_LoaderTest):
       d.create_file("path/to/__init__.pyi", "to = ...  # type: str")
       d.create_file("path/to/some/__init__.pyi", "some = ...  # type: str")
       d.create_file("path/to/some/module.pyi", "")
-      loader = load_pytd.Loader("path.to.some.module",
-                                self.python_version,
-                                pythonpath=[d.path])
+      loader = load_pytd.Loader(config.Options.create(
+          module_name="path.to.some.module", python_version=self.python_version,
+          pythonpath=d.path))
       some = loader.import_relative(1)
       to = loader.import_relative(2)
       path = loader.import_relative(3)
@@ -197,7 +211,8 @@ class ImportPathsTest(_LoaderTest):
       self.assertTrue(path.Lookup("path.path"))
 
   def test_typeshed(self):
-    loader = load_pytd.Loader("base", self.python_version)
+    loader = load_pytd.Loader(config.Options.create(
+        module_name="base", python_version=self.python_version))
     self.assertTrue(loader.import_name("urllib.request"))
 
   def test_prefer_typeshed(self):
@@ -205,7 +220,9 @@ class ImportPathsTest(_LoaderTest):
       # Override two modules from typeshed
       d.create_file("typing_extensions/__init__.pyi", "foo: str = ...")
       d.create_file("crypt/__init__.pyi", "foo: str = ...")
-      loader = load_pytd.Loader("x", self.python_version, pythonpath=[d.path])
+      loader = load_pytd.Loader(config.Options.create(
+          module_name="x", python_version=self.python_version,
+          pythonpath=d.path))
       # typing_extensions should ignore the override, crypt should not.
       ast1 = loader.import_name("typing_extensions")
       ast2 = loader.import_name("crypt")
@@ -225,8 +242,9 @@ class ImportPathsTest(_LoaderTest):
       d.create_file("module2.pyi", """
           def f() -> module1.x: ...
       """)
-      loader = load_pytd.Loader(
-          "base", self.python_version, pythonpath=[d.path])
+      loader = load_pytd.Loader(config.Options.create(
+          module_name="base", python_version=self.python_version,
+          pythonpath=d.path))
       module2 = loader.import_name("module2")
       f, = module2.Lookup("module2.f").signatures
       self.assertEqual("List[int]", pytd_utils.Print(f.return_type))
@@ -243,10 +261,10 @@ class ImportPathsTest(_LoaderTest):
           "empty1": "/dev/null",
           "empty2": "/dev/null",
       }
-      # We cannot use tweak(imports_info=...) because that doesn't trigger
-      # post-processing and we need an imports_map for the loader.
-      loader = load_pytd.Loader(
-          "base", self.python_version, imports_map=imports_map, pythonpath=[""])
+      loader = load_pytd.Loader(config.Options.create(
+          module_name="base", python_version=self.python_version,
+          pythonpath=""))
+      loader.options.tweak(imports_map=imports_map)
       normal = loader.import_name("foo")
       self.assertEqual("foo", normal.name)
       loader.import_name("bar")  # check that we can resolve against another.foo
@@ -272,8 +290,9 @@ class ImportPathsTest(_LoaderTest):
       d.create_file("pkg/bar.pyi", """
           from .foo import X
           y = ...  # type: X""")
-      loader = load_pytd.Loader(
-          "pkg.bar", self.python_version, pythonpath=[d.path])
+      loader = load_pytd.Loader(config.Options.create(
+          module_name="pkg.bar", python_version=self.python_version,
+          pythonpath=d.path))
       bar = loader.import_name("pkg.bar")
       f = bar.Lookup("pkg.bar.y")
       self.assertEqual("pkg.foo.X", f.type.name)
@@ -288,7 +307,9 @@ class ImportPathsTest(_LoaderTest):
       d.create_file("pkg/sub/bar.pyi", """
           from .foo import X
           y = ...  # type: X""")
-      loader = load_pytd.Loader("pkg", self.python_version, pythonpath=[d.path])
+      loader = load_pytd.Loader(config.Options.create(
+          module_name="pkg", python_version=self.python_version,
+          pythonpath=d.path))
       ast = loader.import_name("pkg.sub")
       self.assertTrue(ast.Lookup("pkg.sub.X"))
 
@@ -304,14 +325,17 @@ class ImportPathsTest(_LoaderTest):
           from .baz import X""")
       d.create_file("pkg/sub/baz.pyi", """
           class X: ...""")
-      loader = load_pytd.Loader("pkg", self.python_version, pythonpath=[d.path])
+      loader = load_pytd.Loader(config.Options.create(
+          module_name="pkg", python_version=self.python_version,
+          pythonpath=d.path))
       ast = loader.import_name("pkg.sub")
       self.assertTrue(ast.Lookup("pkg.sub.X"))
 
   def test_get_resolved_modules(self):
     with file_utils.Tempdir() as d:
       filename = d.create_file("dir/module.pyi", "def foo() -> str: ...")
-      loader = load_pytd.Loader(None, self.python_version, pythonpath=[d.path])
+      loader = load_pytd.Loader(config.Options.create(
+          python_version=self.python_version, pythonpath=d.path))
       ast = loader.import_name("dir.module")
       modules = loader.get_resolved_modules()
       self.assertEqual(set(modules), {"builtins", "typing", "dir.module"})
@@ -333,7 +357,8 @@ class ImportPathsTest(_LoaderTest):
         _PathType = bytes
         def samestat(stat1: os2.stat_result) -> bool: ...
       """)
-      loader = load_pytd.Loader(None, self.python_version, pythonpath=[d.path])
+      loader = load_pytd.Loader(config.Options.create(
+          python_version=self.python_version, pythonpath=d.path))
       ast = loader.import_name("os2.path")
       self.assertEqual(ast.Lookup("os2.path._PathType").type.name,
                        "builtins.bytes")
@@ -352,7 +377,8 @@ class ImportPathsTest(_LoaderTest):
         def samestate(stat1: os2.stat_result) -> bool: ...
       """)
       d.create_file("posix2.pyi", "class stat_result: ...")
-      loader = load_pytd.Loader(None, self.python_version, pythonpath=[d.path])
+      loader = load_pytd.Loader(config.Options.create(
+          python_version=self.python_version, pythonpath=d.path))
       # Make sure all three modules were resolved properly.
       loader.import_name("os2")
       loader.import_name("os2.path")
@@ -364,7 +390,8 @@ class ImportPathsTest(_LoaderTest):
         from typing import Union as _UnionT
         x: _UnionT[int, str]
       """)
-      loader = load_pytd.Loader(None, self.python_version, pythonpath=[d.path])
+      loader = load_pytd.Loader(config.Options.create(
+          python_version=self.python_version, pythonpath=d.path))
       ast = loader.import_name("test")
       x = ast.Lookup("test.x")
       self.assertIsInstance(x.type, pytd.UnionType)
@@ -375,7 +402,8 @@ class ImportPathsTest(_LoaderTest):
         from typing import Optional as _OptionalT
         x: _OptionalT[int]
       """)
-      loader = load_pytd.Loader(None, self.python_version, pythonpath=[d.path])
+      loader = load_pytd.Loader(config.Options.create(
+          python_version=self.python_version, pythonpath=d.path))
       ast = loader.import_name("test")
       x = ast.Lookup("test.x")
       self.assertIsInstance(x.type, pytd.UnionType)
@@ -386,7 +414,8 @@ class ImportPathsTest(_LoaderTest):
         from typing import Intersection as _IntersectionT
         x: _IntersectionT[int, str]
       """)
-      loader = load_pytd.Loader(None, self.python_version, pythonpath=[d.path])
+      loader = load_pytd.Loader(config.Options.create(
+          python_version=self.python_version, pythonpath=d.path))
       ast = loader.import_name("test")
       x = ast.Lookup("test.x")
       self.assertIsInstance(x.type, pytd.IntersectionType)
@@ -394,8 +423,9 @@ class ImportPathsTest(_LoaderTest):
   def test_open_function(self):
     def mock_open(*unused_args, **unused_kwargs):
       return io.StringIO("x: int")
-    loader = load_pytd.Loader(
-        "base", self.python_version, open_function=mock_open)
+    loader = load_pytd.Loader(config.Options.create(
+        module_name="base", python_version=self.python_version,
+        open_function=mock_open))
     a = loader.load_file("a", "a.pyi")
     self.assertEqual("int", pytd_utils.Print(a.Lookup("a.x").type))
 
@@ -405,7 +435,8 @@ class ImportPathsTest(_LoaderTest):
       d.create_file("foo/__init__.pyi", """
         from . import bar as bar
       """)
-      loader = load_pytd.Loader(None, self.python_version, pythonpath=[d.path])
+      loader = load_pytd.Loader(config.Options.create(
+          python_version=self.python_version, pythonpath=d.path))
       foo = loader.import_name("foo")
       self.assertEqual(pytd_utils.Print(foo), "import foo.bar")
 
@@ -415,7 +446,8 @@ class ImportPathsTest(_LoaderTest):
       d.create_file("foo/__init__.pyi", """
         from . import bar as baz
       """)
-      loader = load_pytd.Loader(None, self.python_version, pythonpath=[d.path])
+      loader = load_pytd.Loader(config.Options.create(
+          python_version=self.python_version, pythonpath=d.path))
       foo = loader.import_name("foo")
       self.assertEqual(pytd_utils.Print(foo), "from foo import bar as foo.baz")
 
@@ -428,7 +460,8 @@ class ImportPathsTest(_LoaderTest):
         from foo import *
         def f() -> List[int]: ...
       """)
-      loader = load_pytd.Loader(None, self.python_version, pythonpath=[d.path])
+      loader = load_pytd.Loader(config.Options.create(
+          python_version=self.python_version, pythonpath=d.path))
       foo = loader.import_name("foo")
       bar = loader.import_name("bar")
       self.assertEqual(pytd_utils.Print(foo),
@@ -449,7 +482,8 @@ class ImportPathsTest(_LoaderTest):
         from foo import *
         def f(x: Ellipsis): ...
       """)
-      loader = load_pytd.Loader(None, self.python_version, pythonpath=[d.path])
+      loader = load_pytd.Loader(config.Options.create(
+          python_version=self.python_version, pythonpath=d.path))
       loader.import_name("foo")
       bar = loader.import_name("bar")
       self.assertEqual(pytd_utils.Print(bar.Lookup("bar.f")),
@@ -475,7 +509,8 @@ class ImportPathsTest(_LoaderTest):
         from . import Foo
         class Bar(Foo): ...
       """)
-      loader = load_pytd.Loader(None, self.python_version, pythonpath=[d.path])
+      loader = load_pytd.Loader(config.Options.create(
+          python_version=self.python_version, pythonpath=d.path))
       loader.import_name("foo.bar")
 
   def test_module_alias(self):
@@ -577,9 +612,9 @@ class PickledPyiLoaderTest(test_base.UnitTest):
     return os.path.join(tempdir.path, filename)
 
   def _load_ast(self, tempdir, module):
-    loader = load_pytd.Loader(
-        base_module=module.module_name, python_version=self.python_version,
-        pythonpath=[tempdir.path])
+    loader = load_pytd.Loader(config.Options.create(
+        module_name=module.module_name, python_version=self.python_version,
+        pythonpath=tempdir.path))
     return loader, loader.load_file(
         module.module_name, self._get_path(tempdir, module.file_name))
 
@@ -590,9 +625,8 @@ class PickledPyiLoaderTest(test_base.UnitTest):
           self._get_path(tempdir, module.file_name + ".pickled"))
 
   def _load_pickled_module(self, tempdir, module):
-    pickle_loader = load_pytd.PickledPyiLoader(
-        base_module=None, python_version=self.python_version,
-        pythonpath=[tempdir.path])
+    pickle_loader = load_pytd.PickledPyiLoader(config.Options.create(
+        python_version=self.python_version, pythonpath=tempdir.path))
     return pickle_loader.load_file(
         module.module_name, self._get_path(tempdir, module.file_name))
 
@@ -659,13 +693,17 @@ class PickledPyiLoaderTest(test_base.UnitTest):
         tz = ...  # type: datetime.tzinfo
       """)
       # save builtins
-      load_pytd.Loader("base", self.python_version).save_to_pickle(filename)
+      load_pytd.Loader(config.Options.create(
+          module_name="base", python_version=self.python_version
+      )).save_to_pickle(filename)
       # load builtins
       loader = load_pytd.PickledPyiLoader.load_from_pickle(
-          filename, "base",
-          python_version=self.python_version,
-          imports_map={"foo": foo_path},
-          pythonpath=[""])
+          filename,
+          config.Options.create(
+              module_name="base",
+              python_version=self.python_version,
+              pythonpath=""))
+      loader.options.tweak(imports_map={"foo": foo_path})
       # test import
       self.assertTrue(loader.import_name("sys"))
       self.assertTrue(loader.import_name("__future__"))
@@ -831,5 +869,37 @@ class RecursiveAliasTest(_LoaderTest):
     self.assertEqual(actual_z, expected_z)
 
 
+class NestedClassTest(_LoaderTest):
+
+  def test_basic(self):
+    ast = self._import(a="""
+      class A:
+        class B:
+          def f(self) -> C: ...
+        class C(B): ...
+    """)
+    actual_f = pytd.LookupItemRecursive(ast, "A.B.f")
+    self.assertEqual(pytd_utils.Print(actual_f),
+                     "def f(self: a.A.B) -> a.A.C: ...")
+    actual_c = pytd.LookupItemRecursive(ast, "A.C")
+    self.assertEqual(pytd_utils.Print(actual_c).rstrip(),
+                     "class a.A.C(a.A.B): ...")
+
+  @test_base.skip("This does not work yet")
+  def test_shadowing(self):
+    ast = self._import(a="""
+      class A:
+        class A(A):
+          def f(self) -> A: ...
+        class C(A): ...
+    """)
+    self.assertEqual(pytd_utils.Print(ast).rstrip(), textwrap.dedent("""
+      class a.A:
+          class a.A.A(a.A):
+              def f(self) -> a.A.A: ...
+          class a.A.C(a.A.A): ...
+      """).strip())
+
+
 if __name__ == "__main__":
   unittest.main()
