@@ -140,7 +140,9 @@ class Converter(utils.ContextWeakrefMixin):
     Returns:
       A PyTD type.
     """
-    if isinstance(v, abstract.Union):
+    if v.is_late_annotation() and v.is_recursive():
+      return pytd.LateType(v.expr)
+    elif isinstance(v, abstract.Union):
       return pytd.UnionType(tuple(
           self.value_instance_to_pytd_type(node, t, instance, seen, view)
           for t in v.options))
@@ -272,10 +274,7 @@ class Converter(utils.ContextWeakrefMixin):
       return pytd.GenericType(base_type=pytd.NamedType("builtins.type"),
                               parameters=(param,))
     elif isinstance(v, abstract.Module):
-      if self.ctx.options.gen_stub_imports:
-        return pytd.Alias(v.name, pytd.Module(v.name, module_name=v.full_name))
-      else:
-        return pytd.NamedType("builtins.module")
+      return pytd.Alias(v.name, pytd.Module(v.name, module_name=v.full_name))
     elif (self._output_mode >= Converter.OutputMode.LITERAL and
           isinstance(v, abstract.ConcreteValue) and
           isinstance(v.pyval, (int, str, bytes))):
@@ -358,7 +357,7 @@ class Converter(utils.ContextWeakrefMixin):
     Returns:
       A PyTD definition.
     """
-    if self.ctx.options.gen_stub_imports and isinstance(v, abstract.Module):
+    if isinstance(v, abstract.Module):
       return pytd.Alias(name, pytd.Module(name, module_name=v.full_name))
     elif isinstance(v, abstract.BoundFunction):
       d = self.value_to_pytd_def(node, v.underlying, name)
@@ -764,7 +763,7 @@ class Converter(utils.ContextWeakrefMixin):
 
     cls = pytd.Class(name=class_name,
                      metaclass=metaclass,
-                     parents=tuple(bases),
+                     bases=tuple(bases),
                      methods=tuple(methods.values()),
                      constants=tuple(constants),
                      classes=(),
