@@ -112,7 +112,7 @@ class Attrs(classgen.Decorator):
             attr.typ = self.ctx.convert.unsolvable
           elif attrib.type_source == TypeSource.CONVERTER:
             msg = "attr.ib type has been assigned by the converter."
-            self.ctx.vm.check_annotation_type_mismatch(
+            self.ctx.check_annotation_type_mismatch(
                 node,
                 name,
                 typ,
@@ -147,7 +147,7 @@ class Attrs(classgen.Decorator):
         msg = ("Note: The 'assignment' here is the 'default' or 'factory' arg,"
                " which conflicts with the field type (set via annotation or a"
                " 'type' or 'converter' arg).")
-        self.ctx.vm.check_annotation_type_mismatch(
+        self.ctx.check_annotation_type_mismatch(
             node,
             attr.name,
             attr.typ,
@@ -158,7 +158,7 @@ class Attrs(classgen.Decorator):
         own_attrs.append(attr)
       elif self.args[cls]["auto_attribs"]:
         if not match_classvar(typ):
-          self.ctx.vm.check_annotation_type_mismatch(
+          self.ctx.check_annotation_type_mismatch(
               node, name, typ, orig, local.stack, allow_none=True)
           attr = Attribute(
               name=name, typ=typ, init=True, kw_only=False, default=orig)
@@ -184,6 +184,21 @@ class Attrs(classgen.Decorator):
       # Fix up type parameters in methods added by the decorator.
       cls.update_method_type_params()
 
+  def to_metadata(self):
+    return {
+        "tag": "attr.s",
+        "init": self._current_args["init"],
+        "kw_only": self._current_args["kw_only"],
+        "auto_attribs": self._current_args["auto_attribs"]
+    }
+
+  @classmethod
+  def from_metadata(cls, ctx, metadata):
+    kwargs = {k: metadata[k] for k in ("init", "kw_only", "auto_attribs")}
+    ret = cls.make(ctx)
+    ret.set_current_args(kwargs)
+    return ret
+
 
 class AttrsNextGenDefine(Attrs):
   """Implements the @attr.define decorator.
@@ -306,10 +321,31 @@ class Attrib(classgen.FieldConstructor):
   def make(cls, ctx):
     return super().make("ib", ctx, "attr")
 
-  def call(self, node, unused_func, args):
+  def _match_and_discard_args(self, node, funcb, args):
+    """Discard invalid args so that we can still construct an attrib."""
+    func = funcb.data
+    args, errors = function.match_all_args(self.ctx, node, func, args)
+    # Raise the error, but continue constructing the attrib, so that if we
+    # have something like
+    #   x = attr.ib(<bad args>)
+    # we still proceed with the rest of the analysis with x in the list of
+    # attribs, and with our best guess for typing information.
+    for e, name, _ in errors:
+      self.ctx.errorlog.invalid_function_call(self.ctx.vm.stack(func), e)
+      # The presence of 'factory' or 'default' means we need to preserve the
+      # fact that the attrib was intended to have a default, otherwise we might
+      # get an invalid sig for __init__ with a non-default param following a
+      # default param.
+      if name != "default":
+        args = args.delete_namedarg(name)
+      if name == "factory":
+        args = args.replace_namedarg("default", self.ctx.new_unsolvable(node))
+    return args
+
+  def call(self, node, funcb, args):
     """Returns a type corresponding to an attr."""
     args = args.simplify(node, self.ctx)
-    self.match_args(node, args)
+    args = self._match_and_discard_args(node, funcb, args)
     node, default_var = self._get_default_var(node, args)
     type_var = args.namedargs.get("type")
     init = self.get_kwarg(args, "init", True)
@@ -343,7 +379,7 @@ class Attrib(classgen.FieldConstructor):
       if type_source == TypeSource.TYPE:
         msg = ("The type annotation and assignment are set by the "
                "'type' and 'converter' args respectively.")
-        self.ctx.vm.check_annotation_type_mismatch(
+        self.ctx.check_annotation_type_mismatch(
             node,
             "attr.ib",
             typ,
