@@ -3,6 +3,7 @@
 import collections
 import dataclasses
 import re
+import sys
 
 from typing import Any, Dict, List, Optional, Union
 
@@ -19,7 +20,12 @@ from pytype.pytd.codegen import namedtuple
 from pytype.pytd.codegen import pytdgen
 from pytype.pytd.parse import node as pytd_node
 
-from typed_ast import ast3
+# pylint: disable=g-import-not-at-top
+if sys.version_info >= (3, 8):
+  import ast as ast3
+else:
+  from typed_ast import ast3
+# pylint: enable=g-import-not-at-top
 
 
 # Typing members that represent sets of types.
@@ -105,8 +111,6 @@ def _maybe_resolve_alias(alias, name_to_class, name_to_constant):
     if isinstance(value, pytd.Constant):
       if (not isinstance(value.type, pytd.NamedType) or
           value.type.name not in name_to_class):
-        # TODO(rechen): Parameterized constants of generic classes should
-        # probably also be allowed.
         return alias
       value = name_to_class[value.type.name]
     if not isinstance(value, pytd.Class):
@@ -114,13 +118,13 @@ def _maybe_resolve_alias(alias, name_to_class, name_to_constant):
     try:
       value = value.Lookup(part)
     except KeyError:
-      for parent in value.parents:
-        if parent.name not in name_to_class:
-          # If the parent is unknown, we don't know whether it contains 'part',
+      for base in value.bases:
+        if base.name not in name_to_class:
+          # If the base is unknown, we don't know whether it contains 'part',
           # so it cannot be resolved.
           return alias
         try:
-          value = name_to_class[parent.name].Lookup(part)
+          value = name_to_class[base.name].Lookup(part)
         except KeyError:
           continue  # continue up the MRO
         else:
@@ -147,7 +151,7 @@ def pytd_literal(parameters: List[Any]) -> pytd.Type:
     elif isinstance(p, pytd.NamedType):
       # TODO(b/173742489): support enums.
       literal_parameters.append(pytd.AnythingType())
-    elif isinstance(p, types.Constant):
+    elif isinstance(p, types.Pyval):
       literal_parameters.append(p.to_pytd_literal())
     elif isinstance(p, pytd.Literal):
       literal_parameters.append(p)
@@ -164,7 +168,7 @@ def pytd_literal(parameters: List[Any]) -> pytd.Type:
 
 def _convert_annotated(x):
   """Convert everything to a string to store it in pytd.Annotated."""
-  if isinstance(x, types.Constant):
+  if isinstance(x, types.Pyval):
     return x.repr_str()
   elif isinstance(x, dict):
     return metadata.to_string(x)
@@ -220,7 +224,7 @@ class _VerifyMutators(visitors.Visitor):
 
   def EnterClass(self, node):
     params = set()
-    for cls in node.parents:
+    for cls in node.bases:
       params |= self._GetTypeParameters(cls)
     self._AddParams(params)
 
@@ -362,7 +366,7 @@ class Definitions:
         name, len(self.generated_classes[name]))
     cls = pytd.Class(name=cls_name,
                      metaclass=None,
-                     parents=(typ,),
+                     bases=(typ,),
                      methods=tuple(methods),
                      constants=(),
                      decorators=(),
@@ -496,9 +500,9 @@ class Definitions:
       # We do not yet support PEP 647, User-Defined Type Guards. To avoid
       # blocking typeshed, convert type guards to plain bools.
       return pytd.NamedType("bool")
-    elif any(isinstance(p, types.Constant) for p in parameters):
+    elif any(isinstance(p, types.Pyval) for p in parameters):
       parameters = ", ".join(
-          p.repr_str() if isinstance(p, types.Constant) else "_"
+          p.repr_str() if isinstance(p, types.Pyval) else "_"
           for p in parameters)
       raise ParseError(
           "%s[%s] not supported" % (pytd_utils.Print(base_type), parameters))
@@ -616,8 +620,8 @@ class Definitions:
       self, class_name, bases, keywords, decorators, defs
   ) -> pytd.Class:
     """Build a pytd.Class from definitions collected from an ast node."""
-    parents, namedtuple_index = classdef.get_parents(bases)
-    metaclass = classdef.get_metaclass(keywords, parents)
+    bases, namedtuple_index = classdef.get_bases(bases)
+    metaclass = classdef.get_metaclass(keywords, bases)
     constants, methods, aliases, slots, classes = _split_definitions(defs)
 
     # Make sure we don't have duplicate definitions.
@@ -625,9 +629,9 @@ class Definitions:
 
     # Generate a NamedTuple proxy base class if needed
     if namedtuple_index is not None:
-      namedtuple_parent = self.new_named_tuple(
+      namedtuple_base = self.new_named_tuple(
           class_name, [(c.name, c.type) for c in constants])
-      parents[namedtuple_index] = namedtuple_parent
+      bases[namedtuple_index] = namedtuple_base
       constants = []
 
     if aliases:
@@ -665,16 +669,16 @@ class Definitions:
             t = val.type
           constants.append(pytd.Constant(name, t))
 
-    parents = [p for p in parents if not isinstance(p, pytd.NothingType)]
+    bases = [p for p in bases if not isinstance(p, pytd.NothingType)]
     methods = function.merge_method_signatures(methods)
-    if not parents and class_name not in ["classobj", "object"]:
-      # A parent-less class inherits from classobj in Python 2 and from object
+    if not bases and class_name not in ["classobj", "object"]:
+      # A bases-less class inherits from classobj in Python 2 and from object
       # in Python 3. typeshed assumes the Python 3 behavior for all stubs, so we
       # do the same here.
-      parents = (pytd.NamedType("object"),)
+      bases = (pytd.NamedType("object"),)
 
     return pytd.Class(name=class_name, metaclass=metaclass,
-                      parents=tuple(parents),
+                      bases=tuple(bases),
                       methods=tuple(methods),
                       constants=tuple(constants),
                       classes=tuple(classes),
@@ -699,6 +703,7 @@ class Definitions:
 
     classes = generated_classes + classes
     functions = function.merge_method_signatures(functions)
+    _check_module_functions(functions)
 
     name_to_class = {c.name: c for c in classes}
     name_to_constant = {c.name: c for c in constants}
@@ -727,12 +732,6 @@ class Definitions:
       raise ParseError(
           "Duplicate top-level identifier(s): " + ", ".join(duplicates))
 
-    properties = [x for x in functions if x.kind == pytd.MethodTypes.PROPERTY]
-    if properties:
-      prop_names = ", ".join(p.name for p in properties)
-      raise ParseError(
-          "Module-level functions with property decorators: " + prop_names)
-
     return pytd.TypeDeclUnit(name=None,
                              constants=tuple(constants),
                              type_params=tuple(self.type_params),
@@ -746,3 +745,18 @@ def finalize_ast(ast: pytd.TypeDeclUnit):
   ast = ast.Visit(_InsertTypeParameters(ast.type_params))
   ast = ast.Visit(_VerifyMutators())
   return ast
+
+
+def _check_module_functions(functions):
+  """Validate top-level module functions."""
+  # module.__getattr__ should have a unique signature
+  g = [f for f in functions if f.name == "__getattr__"]
+  if g and len(g[0].signatures) > 1:
+    raise ParseError("Multiple signatures for module __getattr__")
+
+  # module-level functions cannot be properties
+  properties = [x for x in functions if x.kind == pytd.MethodTypes.PROPERTY]
+  if properties:
+    prop_names = ", ".join(p.name for p in properties)
+    raise ParseError(
+        "Module-level functions with property decorators: " + prop_names)
