@@ -6,6 +6,7 @@ from pytype.pyi import parser
 from pytype.pyi import parser_test_base
 from pytype.pytd import pytd
 from pytype.tests import test_base
+from pytype.tests import test_utils
 
 import unittest
 
@@ -90,7 +91,8 @@ class ParserTest(parser_test_base.ParserTestBase):
     self.check_error("x = 12.3", 1,
                      "Only '0.0' allowed as float literal")
 
-  def test_string_constant(self):
+  @test_utils.skipFromPy((3, 8), "No longer tagging strings 'unicode'")
+  def test_string_constant_37(self):
     self.check("x = b''", "x: bytes")
     self.check("x = u''", "x: unicode")
     self.check('x = b""', "x: bytes")
@@ -98,6 +100,15 @@ class ParserTest(parser_test_base.ParserTestBase):
     self.check("x = ''", "x: str")
     self.check('x = ""', "x: str")
 
+  @test_utils.skipBeforePy((3, 8), "No longer tagging strings 'unicode'")
+  def test_string_constant_38(self):
+    self.check("x = b''", "x: bytes")
+    self.check("x = u''", "x: str")
+    self.check('x = b""', "x: bytes")
+    self.check('x = u""', "x: str")
+    self.check("x = ''", "x: str")
+    self.check('x = ""', "x: str")
+
   @unittest.skip("We allow all strings.")
   def test_invalid_string_constant(self):
     self.check_error("x = b'x'", 1,
@@ -276,8 +287,8 @@ class ParserTest(parser_test_base.ParserTestBase):
   def test_from_import(self):
     ast = self.check("from foo import c\nclass Bar(c.X): ...",
                      parser_test_base.IGNORE)
-    parent, = ast.Lookup("Bar").parents
-    self.assertEqual(parent, pytd.NamedType("foo.c.X"))
+    base, = ast.Lookup("Bar").bases
+    self.assertEqual(base, pytd.NamedType("foo.c.X"))
 
   def test_duplicate_names(self):
     self.check_error("""
@@ -451,7 +462,7 @@ class ParserTest(parser_test_base.ParserTestBase):
         this is not valid"""
     with self.assertRaises(parser.ParseError) as e:
       parser.parse_string(textwrap.dedent(src).lstrip(), filename="foo.py",
-                          python_version=self.python_version)
+                          options=self.options)
     self.assertMultiLineEqual(textwrap.dedent("""
         File: "foo.py", line 2
           this is not valid
@@ -492,7 +503,7 @@ class ParserTest(parser_test_base.ParserTestBase):
         X = ... # type: Any
       y = bar.X.Baz
       z = X.Baz
-    """), name="foo", python_version=self.python_version)
+    """), name="foo", options=self.options)
     self.assertEqual("foo.bar.X.Baz", ast.Lookup("foo.y").type.name)
     self.assertEqual("bar.X.Baz", ast.Lookup("foo.z").type.name)
 
@@ -522,6 +533,16 @@ class ParserTest(parser_test_base.ParserTestBase):
       __all__: List[str]
     """)
 
+  def test_invalid_constructor(self):
+    e = "Constructors and function calls in type annotations are not supported."
+    self.check_error("""
+      x = ... # type: typing.NamedTuple("A", [])
+    """, 1, e)
+
+    self.check_error("""
+      x: typing.NamedTuple("A", []) = ...
+    """, 1, e)
+
 
 class QuotedTypeTest(parser_test_base.ParserTestBase):
 
@@ -656,101 +677,6 @@ class HomogeneousTypeTest(parser_test_base.ParserTestBase):
 
 class NamedTupleTest(parser_test_base.ParserTestBase):
 
-  @unittest.skip("Constructors in type annotations not supported")
-  def test_no_fields(self):
-    self.check("x = ...  # type: NamedTuple('foo', [])", """
-      from typing import Any, Tuple, Type, TypeVar
-
-      x: namedtuple_foo_0
-
-      _Tnamedtuple_foo_0 = TypeVar('_Tnamedtuple_foo_0', bound=namedtuple_foo_0)
-
-      class namedtuple_foo_0(Tuple[()]):
-          __slots__ = []
-          _asdict: Any
-          __dict__: Any
-          _fields: Any
-          __getnewargs__: Any
-          __getstate__: Any
-          _make: Any
-          _replace: Any
-          def __new__(cls: Type[_Tnamedtuple_foo_0]) -> _Tnamedtuple_foo_0: ...
-          def __init__(self, *args, **kwargs) -> None: ...
-      """)
-
-  @unittest.skip("Constructors in type annotations not supported")
-  def test_multiple_fields(self):
-    expected = """
-      from typing import Any, Tuple, Type, TypeVar
-
-      x: namedtuple_foo_0
-
-      _Tnamedtuple_foo_0 = TypeVar('_Tnamedtuple_foo_0', bound=namedtuple_foo_0)
-
-      class namedtuple_foo_0(Tuple[int, str]):
-          __slots__ = ["a", "b"]
-          a: int
-          b: str
-          _asdict: Any
-          __dict__: Any
-          _fields: Any
-          __getnewargs__: Any
-          __getstate__: Any
-          _make: Any
-          _replace: Any
-          def __new__(cls: Type[_Tnamedtuple_foo_0], a: int, b: str) -> _Tnamedtuple_foo_0: ...
-          def __init__(self, *args, **kwargs) -> None: ...
-    """
-    self.check("x = ...  # type: NamedTuple('foo', [('a', int), ('b', str)])",
-               expected)
-    self.check("x = ...  # type: NamedTuple('foo', [('a', int), ('b', str),])",
-               expected)
-    self.check("x = ...  # type: NamedTuple('foo', [('a', int,), ('b', str),])",
-               expected)
-
-  # pylint: disable=line-too-long
-  @unittest.skip("Constructors in type annotations not supported")
-  def test_dedup_basename(self):
-    self.check("""
-      x = ...  # type: NamedTuple('foo', [('a', int,)])
-      y = ...  # type: NamedTuple('foo', [('b', str,)])""",
-               """
-      from typing import Any, Tuple, Type, TypeVar
-
-      x: namedtuple_foo_0
-      y: namedtuple_foo_1
-
-      _Tnamedtuple_foo_0 = TypeVar('_Tnamedtuple_foo_0', bound=namedtuple_foo_0)
-      _Tnamedtuple_foo_1 = TypeVar('_Tnamedtuple_foo_1', bound=namedtuple_foo_1)
-
-      class namedtuple_foo_0(Tuple[int]):
-          __slots__ = ["a"]
-          a: int
-          _asdict: Any
-          __dict__: Any
-          _fields: Any
-          __getnewargs__: Any
-          __getstate__: Any
-          _make: Any
-          _replace: Any
-          def __new__(cls: Type[_Tnamedtuple_foo_0], a: int) -> _Tnamedtuple_foo_0: ...
-          def __init__(self, *args, **kwargs) -> None: ...
-
-      class namedtuple_foo_1(Tuple[str]):
-          __slots__ = ["b"]
-          b: str
-          _asdict: Any
-          __dict__: Any
-          _fields: Any
-          __getnewargs__: Any
-          __getstate__: Any
-          _make: Any
-          _replace: Any
-          def __new__(cls: Type[_Tnamedtuple_foo_1], b: str) -> _Tnamedtuple_foo_1: ...
-          def __init__(self, *args, **kwargs) -> None: ...
-        """)
-  # pylint: enable=line-too-long
-
   def test_assign_namedtuple(self):
     self.check("X = NamedTuple('X', [])", """
       from typing import Any, Tuple, Type, TypeVar
@@ -975,7 +901,7 @@ class NamedTupleTest(parser_test_base.ParserTestBase):
       class X(dict, namedtuple_X_0): ...
     """)
 
-  def test_multi_namedtuple_parent(self):
+  def test_multi_namedtuple_base(self):
     self.check_error("""
       from typing import NamedTuple
       class X(NamedTuple, NamedTuple): ...
@@ -1209,13 +1135,24 @@ class FunctionTest(parser_test_base.ParserTestBase):
                      None,
                      "Module-level functions with property decorators: foo")
 
+    line = 1 if sys.version_info < (3, 8) else 3
     self.check_error("""
       @classmethod
       @staticmethod
       def foo() -> int: ...""",
-                     1,
+                     line,
                      "Too many decorators for foo")
 
+  def test_module_getattr(self):
+    self.check("""
+      def __getattr__(name) -> int: ...
+    """)
+
+    self.check_error("""
+      def __getattr__(name) -> int: ...
+      def __getattr__(name) -> str: ...
+    """, None, "Multiple signatures for module __getattr__")
+
   def test_type_check_only(self):
     self.check("""
       from typing import type_check_only
@@ -1244,10 +1181,11 @@ class FunctionTest(parser_test_base.ParserTestBase):
     """)
 
   def test_bad_decorated_class(self):
+    line = 1 if sys.version_info < (3, 8) else 2
     self.check_error("""
       @classmethod
       class Foo: ...
-    """, 1, "Unsupported class decorators: classmethod")
+    """, line, "Unsupported class decorators: classmethod")
 
   def test_dataclass_decorator(self):
     self.check("""
@@ -1359,7 +1297,7 @@ class FunctionTest(parser_test_base.ParserTestBase):
 
 class ClassTest(parser_test_base.ParserTestBase):
 
-  def test_no_parents(self):
+  def test_no_bases(self):
     canonical = """
       class Foo: ...
       """
@@ -1370,7 +1308,7 @@ class ClassTest(parser_test_base.ParserTestBase):
           pass
       """, canonical)
 
-  def test_parents(self):
+  def test_bases(self):
     self.check("""
       class Foo(Bar): ...
     """)
@@ -1378,7 +1316,7 @@ class ClassTest(parser_test_base.ParserTestBase):
       class Foo(Bar, Baz): ...
       """)
 
-  def test_parent_remove_nothingtype(self):
+  def test_base_remove_nothingtype(self):
     self.check("""
       class Foo(nothing): ...
       """, """
@@ -1511,14 +1449,14 @@ class ClassTest(parser_test_base.ParserTestBase):
           def x(self) -> str: ...
       """)
 
-  def test_protocol_parent(self):
+  def test_protocol_base(self):
     self.check("""
       from typing import Protocol
 
       class Foo(Protocol): ...
     """)
 
-  def test_parameterized_protocol_parent(self):
+  def test_parameterized_protocol_base(self):
     self.check("""
       from typing import Protocol, TypeVar
 
@@ -2052,6 +1990,7 @@ class PropertyDecoratorTest(parser_test_base.ParserTestBase):
       """, expected)
 
   def test_property_decorator_bad_syntax(self):
+    line = 1 if sys.version_info < (3, 8) else 2
     self.check_error("""
       class A:
           @property
@@ -2064,12 +2003,13 @@ class PropertyDecoratorTest(parser_test_base.ParserTestBase):
           def name(self): ...
       """, 1, "@name.setter needs 2 param(s), got 1")
 
+    line = 2 if sys.version_info < (3, 8) else 4
     self.check_error("""
       class A:
           @property
           @staticmethod
           def name(self): ...
-      """, 2, "Too many decorators for name")
+      """, line, "Too many decorators for name")
 
     self.check_error("""
       @property
@@ -2086,6 +2026,21 @@ class PropertyDecoratorTest(parser_test_base.ParserTestBase):
           def name(self) -> int: ...
       """, 1, "Invalid property decorators for method `name`")
 
+  def test_abstract_property(self):
+    self.check("""
+      class Foo:
+        @property
+        @abstractmethod
+        def x(self) -> int: ...
+        @x.setter
+        def x(self, y: int) -> None: ...
+    """, """
+      from typing import Annotated
+
+      class Foo:
+          x: Annotated[int, 'property']
+    """)
+
 
 class MergeSignaturesTest(parser_test_base.ParserTestBase):
 
@@ -2251,7 +2206,7 @@ class CanonicalPyiTest(parser_test_base.ParserTestBase):
         def foo(x: str) -> Any: ...
     """).strip()
     self.assertMultiLineEqual(
-        parser.canonical_pyi(src, self.python_version), expected)
+        parser.canonical_pyi(src, options=self.options), expected)
 
 
 class TypeMacroTest(parser_test_base.ParserTestBase):
@@ -2386,7 +2341,7 @@ class ImportTypeIgnoreTest(parser_test_base.ParserTestBase):
       from mod import attr  # type: ignore
       def f(x: attr) -> None: ...
     """)
-    ast = parser.parse_string(src, python_version=self.python_version)
+    ast = parser.parse_string(src, options=self.options)
     self.assertTrue(ast.Lookup("attr"))
     self.assertTrue(ast.Lookup("f"))
 
@@ -2395,7 +2350,7 @@ class ImportTypeIgnoreTest(parser_test_base.ParserTestBase):
       from . import attr  # type: ignore
       def f(x: attr) -> None: ...
     """)
-    ast = parser.parse_string(src, python_version=self.python_version)
+    ast = parser.parse_string(src, options=self.options)
     self.assertTrue(ast.Lookup("attr"))
     self.assertTrue(ast.Lookup("f"))
 
@@ -2404,7 +2359,7 @@ class ImportTypeIgnoreTest(parser_test_base.ParserTestBase):
       from .. import attr  # type: ignore
       def f(x: attr) -> None: ...
     """)
-    ast = parser.parse_string(src, python_version=self.python_version)
+    ast = parser.parse_string(src, options=self.options)
     self.assertTrue(ast.Lookup("attr"))
     self.assertTrue(ast.Lookup("f"))
 
@@ -2443,6 +2398,7 @@ class LiteralTest(parser_test_base.ParserTestBase):
       z: Literal[b'xyz']
     """)
 
+  @test_utils.skipFromPy((3, 8), "No longer tagging strings 'unicode'")
   def test_unicodestring(self):
     self.check("""
       from typing import Literal
@@ -2482,11 +2438,11 @@ class LiteralTest(parser_test_base.ParserTestBase):
     self.check("""
       from typing import Literal
 
-      x: Literal[True, 0, b"", u"", None]
+      x: Literal[True, 0, b"", "", None]
     """, """
       from typing import Literal, Optional
 
-      x: Optional[Literal[True, 0, b'', u'']]
+      x: Optional[Literal[True, 0, b'', '']]
     """)
 
   def test_stray_number(self):
@@ -2510,6 +2466,7 @@ class LiteralTest(parser_test_base.ParserTestBase):
       x: Tuple[str, b'', str, str]
     """, 3, "Tuple[_, b'', _, _] not supported")
 
+  @test_utils.skipFromPy((3, 8), "No longer tagging strings 'unicode'")
   def test_stray_unicodestring(self):
     self.check_error("""
       from typing import Tuple
@@ -2789,7 +2746,7 @@ class ErrorTest(test_base.UnitTest):
       a: int
     """)
     with self.assertRaisesRegex(parser.ParseError, "File.*foo.pyi"):
-      parser.parse_pyi(src, "foo.pyi", "foo", (3, 6))
+      parser.parse_pyi(src, "foo.pyi", "foo")
 
   def test_lineno(self):
     src = textwrap.dedent("""
@@ -2797,7 +2754,7 @@ class ErrorTest(test_base.UnitTest):
         __slots__ = 0
     """)
     with self.assertRaisesRegex(parser.ParseError, "line 3"):
-      parser.parse_pyi(src, "foo.py", "foo", (3, 6))
+      parser.parse_pyi(src, "foo.py", "foo")
 
 
 class ParamsTest(test_base.UnitTest):
@@ -2805,7 +2762,6 @@ class ParamsTest(test_base.UnitTest):
 
   def test_feature_version(self):
     cases = [
-        [3, sys.version_info.minor],
         [(3,), sys.version_info.minor],
         [(3, 7), 7],
         [(3, 8, 2), 8]
