@@ -1,6 +1,7 @@
 import re
 import textwrap
 
+from pytype import config
 from pytype import load_pytd
 from pytype.pytd import optimize
 from pytype.pytd import pytd
@@ -25,7 +26,8 @@ class TestOptimize(parser_test_base.ParserTest):
   @classmethod
   def setUpClass(cls):
     super(TestOptimize, cls).setUpClass()
-    cls.loader = load_pytd.Loader(None, cls.python_version)
+    cls.loader = load_pytd.Loader(
+        config.Options.create(python_version=cls.python_version))
     cls.builtins = cls.loader.builtins
     cls.typing = cls.loader.typing
 
@@ -89,231 +91,6 @@ class TestOptimize(parser_test_base.ParserTest):
     """)
     self.AssertOptimizeEquals(src, new_src)
 
-  def test_remove_redundant_signature(self):
-    src = pytd_src("""
-        def foo(a: int) -> int: ...
-        def foo(a: Union[int, bool]) -> int: ...
-    """)
-    expected = pytd_src("""
-        def foo(a: Union[int, bool]) -> int: ...
-    """)
-    ast = self.Parse(src)
-    ast = ast.Visit(optimize.RemoveRedundantSignatures(
-        optimize.SuperClassHierarchy({})))
-    self.AssertSourceEquals(ast, expected)
-
-  def test_remove_redundant_signature_with_exceptions(self):
-    src = pytd_src("""
-        def foo(a: int) -> int:
-          raise IOError()
-        def foo(a: Union[int, bool]) -> int: ...
-    """)
-    expected = pytd_src("""
-        def foo(a: int) -> int:
-          raise IOError()
-        def foo(a: Union[int, bool]) -> int: ...
-    """)
-    ast = self.Parse(src)
-    ast = ast.Visit(optimize.RemoveRedundantSignatures(
-        optimize.SuperClassHierarchy({})))
-    self.AssertSourceEquals(ast, expected)
-
-  def test_remove_redundant_signature_with_subclasses(self):
-    src = pytd_src("""
-        def foo(a: bool) -> int: ...
-        def foo(a: int) -> int: ...
-    """)
-    new_src = pytd_src("""
-        def foo(a: int) -> int: ...
-    """)
-    ast = self.ParseAndResolve(src)
-    self.AssertOptimizeEquals(ast, new_src)
-
-  def test_remove_redundant_signature_with_any1(self):
-    src = pytd_src("""
-        from typing import Any
-        def foo(a: Any) -> Any: ...
-        def foo(a: int) -> int: ...
-    """)
-    new_src = pytd_src("""
-        from typing import Any
-        def foo(a) -> Any: ...
-    """)
-    ast = self.ParseAndResolve(src)
-    self.AssertOptimizeEquals(ast, new_src)
-
-  def test_remove_redundant_signature_with_any2(self):
-    src = pytd_src("""
-        from typing import Any
-        def foo(a: int) -> int: ...
-        def foo(a: Any) -> Any: ...
-    """)
-    new_src = pytd_src("""
-        from typing import Any
-        def foo(a) -> Any: ...
-    """)
-    ast = self.ParseAndResolve(src)
-    self.AssertOptimizeEquals(ast, new_src)
-
-  def test_remove_redundant_signature_with_optional1(self):
-    src = pytd_src("""
-        from typing import Any
-        def foo(a: int = ...) -> int: ...
-        def foo(a: Any = ...) -> int: ...
-    """)
-    new_src = pytd_src("""
-        def foo(a = ...) -> int: ...
-    """)
-    ast = self.ParseAndResolve(src)
-    self.AssertOptimizeEquals(ast, new_src)
-
-  def test_remove_redundant_signature_with_optional2(self):
-    src = pytd_src("""
-        from typing import Any
-        def foo(a: Any = ...) -> int: ...
-        def foo(a: int = ...) -> int: ...
-    """)
-    new_src = pytd_src("""
-        def foo(a = ...) -> int: ...
-    """)
-    ast = self.ParseAndResolve(src)
-    self.AssertOptimizeEquals(ast, new_src)
-
-  def test_remove_redundant_signatures_with_long_union(self):
-    src = pytd_src("""
-        from typing import Union
-        def foo(x: None = ...) -> None: ...
-        def foo(x: Union[int, complex, float, long, set, list, unicode,
-                         dict, tuple, str, module, OSError, bytearray,
-                         KeyError, slice, None]) -> None: ...
-    """)
-    new_src = pytd_src("""
-        def foo(x:None = ...) -> None: ...
-        def foo(x) -> None: ...
-    """)
-    ast = self.ParseAndResolve(src)
-    self.AssertOptimizeEquals(ast, new_src)
-
-  def test_remove_redundant_signature_with_object(self):
-    src = pytd_src("""
-        from typing import Any
-        def foo(x) -> Any: ...
-        def foo(x: int) -> int: ...
-    """)
-    new_src = pytd_src("""
-        from typing import Any
-        def foo(x) -> Any: ...
-    """)
-    ast = self.ParseAndResolve(src)
-    self.AssertOptimizeEquals(ast, new_src)
-
-  def test_remove_redundant_signature_any_type(self):
-    src = pytd_src("""
-        from typing import Any
-        def foo(a: int) -> int: ...
-        def foo(a: Any) -> int: ...
-        def bar(a: Any) -> int: ...
-        def bar(a: int) -> int: ...
-        def baz(a: Any) -> Any: ...
-        def baz(a: int) -> int: ...
-        def two(a: Any) -> int: ...
-        def two(a: int) -> Any: ...
-    """)
-    new_src = pytd_src("""
-        from typing import Any
-        def foo(a) -> int: ...
-        def bar(a) -> int: ...
-        def baz(a) -> Any: ...
-        def two(a) -> int: ...
-        def two(a: int) -> Any: ...
-    """)
-    self.AssertOptimizeEquals(src, new_src)
-
-  def test_remove_redundant_signature_generic(self):
-    src = pytd_src("""
-        def foo(a: list[int]) -> list[int]: ...
-        def foo(a: list) -> list: ...
-        def bar(a: list) -> list: ...
-        def bar(a: list[int]) -> list[int]: ...
-    """)
-    new_src = pytd_src("""
-        def foo(a: list) -> list: ...
-        def bar(a: list) -> list: ...
-    """)
-    self.AssertOptimizeEquals(src, new_src)
-
-  def test_remove_redundant_signature_template(self):
-    src = pytd_src("""
-        T = TypeVar("T")
-        class A(Generic[T]):
-          def foo(a: int) -> int: ...
-          def foo(a: T) -> T: ...
-          def foo(a: Union[int, bool]) -> int: ...
-    """)
-    expected = pytd_src("""
-        T = TypeVar("T")
-        class A(Generic[T]):
-          def foo(a: T) -> T: ...
-          def foo(a: Union[int, bool]) -> int: ...
-    """)
-    ast = self.Parse(src)
-    ast = ast.Visit(optimize.RemoveRedundantSignatures(
-        optimize.SuperClassHierarchy({})))
-    self.AssertSourceEquals(ast, expected)
-
-  def test_remove_redundant_signature_two_type_params(self):
-    src = pytd_src("""
-        X = TypeVar("X")
-        Y = TypeVar("Y")
-        class A(Generic[X, Y]):
-          def foo(a: X) -> Y: ...
-          def foo(a: Y) -> Y: ...
-    """)
-    expected = pytd_src("""
-        X = TypeVar("X")
-        Y = TypeVar("Y")
-        class A(Generic[X, Y]):
-          def foo(a: X) -> Y: ...
-          def foo(a: Y) -> Y: ...
-    """)
-    ast = self.Parse(src)
-    ast = ast.Visit(optimize.RemoveRedundantSignatures(
-        optimize.SuperClassHierarchy({})))
-    self.AssertSourceEquals(ast, expected)
-
-  @unittest.skip("Not supported yet.")
-  def test_remove_redundant_signature_generic_left_side(self):
-    src = pytd_src("""
-        from typing import Any
-        X = TypeVar("X")
-        def foo(a: X, b: int) -> X: ...
-        def foo(a: X, b: Any) -> X: ...
-    """)
-    expected = pytd_src("""
-        from typing import Any
-        X = TypeVar("X")
-        def foo(a: X, b: Any) -> X: ...
-    """)
-    ast = self.Parse(src)
-    ast = ast.Visit(optimize.RemoveRedundantSignatures(
-        optimize.SuperClassHierarchy({})))
-    self.AssertSourceEquals(ast, expected)
-
-  def test_remove_redundant_signature_polymorphic(self):
-    src = pytd_src("""
-        T = TypeVar("T")
-        def foo(a: T) -> T: ...
-        def foo(a: Union[int, bool]) -> int: ...
-    """)
-    expected = pytd_src("""
-        T = TypeVar("T")
-        def foo(a: T) -> T: ...
-    """)
-    ast = self.Parse(src)
-    ast = ast.Visit(optimize.RemoveRedundantSignatures(
-        optimize.SuperClassHierarchy({})))
-    self.AssertSourceEquals(ast, expected)
-
   def test_combine_returns(self):
     src = pytd_src("""
         def foo(a: int) -> int: ...
@@ -489,21 +266,6 @@ class TestOptimize(parser_test_base.ParserTest):
     self.AssertSourceEquals(
         self.ApplyVisitorToString(src, optimize.Factorize()), new_src)
 
-  def test_optional_arguments(self):
-    src = pytd_src("""
-        def foo(a: A, *args) -> Z: ...
-        def foo(a: A) -> Z: ...
-        def foo(a: A, b: B) -> Z: ...
-        def foo(a: A, b: B, *args) -> Z: ...
-        def foo() -> Z: ...
-    """)
-    expected = pytd_src("""
-        def foo(a: A, *args) -> Z: ...
-        def foo() -> Z: ...
-    """)
-    new_src = self.ApplyVisitorToString(src, optimize.ApplyOptionalArguments())
-    self.AssertSourceEquals(new_src, expected)
-
   def test_builtin_superclasses(self):
     src = pytd_src("""
         def f(x: Union[list, object], y: Union[complex, memoryview]) -> Union[int, bool]: ...
