@@ -21,7 +21,7 @@ class PrintVisitor(base_visitor.Visitor):
   _RESERVED = frozenset(parser_constants.RESERVED +
                         parser_constants.RESERVED_PYTHON)
 
-  def __init__(self, multiline_args=False):
+  def __init__(self, multiline_args=False, fix_module_collisions=True):
     super().__init__()
     self.class_names = []  # allow nested classes
     self.imports = collections.defaultdict(set)
@@ -31,6 +31,7 @@ class PrintVisitor(base_visitor.Visitor):
     self.in_constant = False
     self.in_signature = False
     self.multiline_args = multiline_args
+    self._fix_module_collisions = fix_module_collisions
 
     self._unit = None
     self._local_names = {}
@@ -258,14 +259,14 @@ class PrintVisitor(base_visitor.Visitor):
 
   def VisitClass(self, node):
     """Visit a class, producing a multi-line, properly indented string."""
-    parents = node.parents
-    # If object is the only parent, we don't need to list any parents.
-    if parents == ("object",):
-      parents = ()
+    bases = node.bases
+    # If object is the only base, we don't need to list any bases.
+    if bases == ("object",):
+      bases = ()
     if node.metaclass is not None:
-      parents += ("metaclass=" + node.metaclass,)
-    parents_str = f"({', '.join(parents)})" if parents else ""
-    header = [f"class {node.name}{parents_str}:"]
+      bases += ("metaclass=" + node.metaclass,)
+    bases_str = f"({', '.join(bases)})" if bases else ""
+    header = [f"class {node.name}{bases_str}:"]
     if node.slots is not None:
       slots_str = ", ".join(f"\"{s}\"" for s in node.slots)
       slots = [self.INDENT + f"__slots__ = [{slots_str}]"]
@@ -441,13 +442,14 @@ class PrintVisitor(base_visitor.Visitor):
   def _GuessModule(self, maybe_module):
     """Guess which part of the given name is the module prefix."""
     if "." not in maybe_module:
-      return maybe_module
+      return maybe_module, ""
     prefix, suffix = maybe_module.rsplit(".", 1)
     # Heuristic: modules are typically lowercase, classes uppercase.
     if suffix[0].islower():
-      return maybe_module
+      return maybe_module, ""
     else:
-      return self._GuessModule(prefix)
+      module, rest = self._GuessModule(prefix)
+      return module, f"{rest}.{suffix}" if rest else suffix
 
   def VisitNamedType(self, node):
     """Convert a type to a string."""
@@ -467,8 +469,20 @@ class PrintVisitor(base_visitor.Visitor):
           if aliased_name:
             node_name = aliased_name
           else:
-            self._RequireImport(self._GuessModule(prefix))
-            node_name = node.name
+            module, rest = self._GuessModule(prefix)
+            if self._fix_module_collisions:
+              module_alias = module
+              while self._NameCollision(module_alias):
+                module_alias = f"_{module_alias}"
+              if module_alias == module:
+                self._RequireImport(module)
+                node_name = node.name
+              else:
+                self._RequireImport(f"{module} as {module_alias}")
+                node_name = ".".join(filter(bool, (module_alias, rest, suffix)))
+            else:
+              self._RequireImport(module)
+              node_name = node.name
         else:
           node_name = node.name
       else:
@@ -503,7 +517,6 @@ class PrintVisitor(base_visitor.Visitor):
 
   def VisitModule(self, node):
     if self.in_constant or self.in_signature:
-      # TODO(slebedev): Use types.ModuleType instead.
       return "module"
     elif not node.is_aliased:
       return f"import {node.module_name}"
