@@ -151,7 +151,7 @@ class Class(Node):
 
   Attributes:
     name: Class name (string)
-    parents: The super classes of this class (instances of pytd.Type).
+    bases: The super classes of this class (instances of pytd.Type).
     methods: Tuple of methods, classmethods, staticmethods
       (instances of pytd.Function).
     constants: Tuple of constant class attributes (instances of pytd.Constant).
@@ -161,7 +161,7 @@ class Class(Node):
   """
   name: str
   metaclass: Union[None, Type]
-  parents: Tuple[Union['Class', Type], ...]
+  bases: Tuple[Union['Class', Type], ...]
   methods: Tuple['Function', ...]
   constants: Tuple[Constant, ...]
   classes: Tuple['Class', ...]
@@ -169,9 +169,6 @@ class Class(Node):
   slots: Optional[Tuple[str, ...]]
   template: Tuple['TemplateItem', ...]
 
-  # TODO(b/159053187): Rename "parents" to "bases". "Parents" is confusing since
-  # we're in a tree.
-
   def Lookup(self, name):
     """Convenience function: Look up a given name in the class namespace.
 
@@ -607,7 +604,7 @@ def IsContainer(t):
   assert isinstance(t, Class)
   if t.name in ('typing.Generic', 'typing.Protocol'):
     return True
-  for p in t.parents:
+  for p in t.bases:
     if isinstance(p, GenericType):
       base = p.base_type
       # We need to check for Generic and Protocol again here because base may
@@ -662,13 +659,17 @@ def ToType(item, allow_constants=False, allow_functions=False,
         return item.type.parameters[0]
       else:
         return AnythingType()
+    elif item.type.name == 'typing_extensions._SpecialForm':
+      # We convert known special forms to their corresponding types and
+      # otherwise treat them as unknown types.
+      if item.name == 'typing_extensions.Protocol':
+        return NamedType('typing.Protocol')
+      else:
+        return AnythingType()
     elif (isinstance(item.type, AnythingType) or
-          item.type.name == 'typing_extensions._SpecialForm' or
           item.name == 'typing_extensions.TypedDict'):
-      # The following constants are treated as (unknown) types:
-      # * one whose type is Any or typing_extensions._SpecialForm
-      # * one whose name is typing_extensions.TypedDict, as TypedDict is a class
-      #   that looks like a constant:
+      # A constant with type Any may be a type, and TypedDict is a class that
+      # looks like a constant:
       #   https://github.com/python/typeshed/blob/8cad322a8ccf4b104cafbac2c798413edaa4f327/third_party/2and3/typing_extensions.pyi#L68
       return AnythingType()
     elif allow_constants:
@@ -749,10 +750,10 @@ def LookupItemRecursive(module, name):
     except KeyError:
       if not isinstance(item, Class):
         raise
-      for parent in item.parents:
-        parent_cls = ExtractClass(parent)  # may raise KeyError
+      for base in item.bases:
+        base_cls = ExtractClass(base)  # may raise KeyError
         try:
-          item = Lookup(parent_cls, lookup_name, part)
+          item = Lookup(base_cls, lookup_name, part)
         except KeyError:
           continue  # continue up the MRO
         else:
