@@ -7,6 +7,9 @@ locally or within a larger repository.
 # len(x) == 0 is clearer in some places:
 # pylint: disable=g-explicit-length-test
 
+# We use a mix of camel case and snake case for method names:
+# pylint: disable=invalid-name
+
 import collections
 import difflib
 import gzip
@@ -28,12 +31,17 @@ from pytype.pytd.parse import parser_constants
 
 _PICKLE_PROTOCOL = pickle.HIGHEST_PROTOCOL
 _PICKLE_RECURSION_LIMIT_AST = 40000
+PICKLE_EXT = ".pickled"
 
 ANON_PARAM = re.compile(r"_[0-9]+")
 
 _TUPLE_NAMES = ("builtins.tuple", "typing.Tuple")
 
 
+def IsPickle(filename):
+  return os.path.splitext(filename)[1].startswith(PICKLE_EXT)
+
+
 def UnpackUnion(t):
   """Return the type list for union type, or a list with the type itself."""
   if isinstance(t, pytd.UnionType):
@@ -111,7 +119,6 @@ def JoinTypes(types):
     return pytd.NothingType()
 
 
-# pylint: disable=invalid-name
 def disabled_function(*unused_args, **unused_kwargs):
   """Disable a function.
 
@@ -177,8 +184,8 @@ def GetAllSubClasses(ast):
   return utils.invert_dict(hierarchy)
 
 
-def Print(ast, multiline_args=False):
-  return ast.Visit(printer.PrintVisitor(multiline_args))
+def Print(ast, multiline_args=False, fix_module_collisions=True):
+  return ast.Visit(printer.PrintVisitor(multiline_args, fix_module_collisions))
 
 
 def CreateModule(name="<empty>", **kwargs):
@@ -453,8 +460,8 @@ def MergeBaseClass(cls, base):
   Returns:
     a pytd.Class of the two merged classes.
   """
-  bases = tuple(b for b in cls.parents if b != base)
-  bases += tuple(b for b in base.parents if b not in bases)
+  bases = tuple(b for b in cls.bases if b != base)
+  bases += tuple(b for b in base.bases if b not in bases)
   method_names = [m.name for m in cls.methods]
   methods = cls.methods + tuple(m for m in base.methods
                                 if m.name not in method_names)
@@ -472,7 +479,7 @@ def MergeBaseClass(cls, base):
     slots = base.slots
   return pytd.Class(name=cls.name,
                     metaclass=cls.metaclass or base.metaclass,
-                    parents=bases,
+                    bases=bases,
                     methods=methods,
                     constants=constants,
                     classes=classes,
