@@ -203,7 +203,7 @@ class TestUtils(parser_test_base.ParserTest):
 
   def test_builtin_alias(self):
     src = "Number = int"
-    ast = parser.parse_string(src, python_version=self.python_version)
+    ast = parser.parse_string(src, options=self.options)
     self.assertMultiLineEqual(pytd_utils.Print(ast), src)
 
   def test_typing_name_conflict1(self):
@@ -214,7 +214,7 @@ class TestUtils(parser_test_base.ParserTest):
 
       def List() -> None: ...
     """)
-    ast = parser.parse_string(src, python_version=self.python_version)
+    ast = parser.parse_string(src, options=self.options)
     self.assertMultiLineEqual(pytd_utils.Print(ast).strip("\n"),
                               src.strip("\n"))
 
@@ -228,7 +228,7 @@ class TestUtils(parser_test_base.ParserTest):
       class MyClass:
           List = ...  # type: Any
           x = ...  # type: typing.List[str]
-    """), python_version=self.python_version)
+    """), options=self.options)
     expected = textwrap.dedent("""
       import typing
       from typing import Any, List
@@ -332,8 +332,8 @@ class TestUtils(parser_test_base.ParserTest):
             def bar(x: T) -> NoneType: ...
         def foo(a: Union[str, int]) -> C: ...
         """)
-    tree1 = parser.parse_string(src1, python_version=self.python_version)
-    tree2 = parser.parse_string(src2, python_version=self.python_version)
+    tree1 = parser.parse_string(src1, options=self.options)
+    tree2 = parser.parse_string(src2, options=self.options)
     tree1.Visit(visitors.VerifyVisitor())
     tree2.Visit(visitors.VerifyVisitor())
     self.assertTrue(tree1.constants)
@@ -372,8 +372,8 @@ class TestUtils(parser_test_base.ParserTest):
     src2 = textwrap.dedent("""
         a: int
         b: float""").lstrip()
-    tree1 = parser.parse_string(src1, python_version=self.python_version)
-    tree2 = parser.parse_string(src2, python_version=self.python_version)
+    tree1 = parser.parse_string(src1, options=self.options)
+    tree2 = parser.parse_string(src2, options=self.options)
     normalize = lambda diff: textwrap.dedent("\n".join(diff))
     self.assertEqual(normalize(pytd_utils.ASTdiff(tree1, tree1)), src1)
     self.assertEqual(normalize(pytd_utils.ASTdiff(tree2, tree2)), src2)
@@ -446,10 +446,9 @@ class PrintTest(parser_test_base.ParserTest):
       from typing import Literal
       x1: Literal[""]
       x2: Literal[b""]
-      x3: Literal[u""]
-      x4: Literal[0]
-      x5: Literal[True]
-      x6: Literal[None]
+      x3: Literal[0]
+      x4: Literal[True]
+      x5: Literal[None]
     """)
     ast = ast.Visit(visitors.LookupBuiltins(self.loader.builtins))
     self.assertMultiLineEqual(pytd_utils.Print(ast), textwrap.dedent("""
@@ -457,10 +456,9 @@ class PrintTest(parser_test_base.ParserTest):
 
       x1: Literal['']
       x2: Literal[b'']
-      x3: Literal[u'']
-      x4: Literal[0]
-      x5: Literal[True]
-      x6: None
+      x3: Literal[0]
+      x4: Literal[True]
+      x5: None
     """).strip())
 
   def test_literal_union(self):
