@@ -258,10 +258,11 @@ def PrepareForExport(module_name, ast, loader):
   # applies transformations,
   # e.g. visitors.PrintVisitor._FormatContainerContents, which need to move to
   # their own visitors so they can be applied without printing.
-  src = pytd_utils.Print(ast)
-  ast = parser.parse_string(src=src, name=module_name,
-                            python_version=loader.python_version,
-                            gen_stub_imports=loader.gen_stub_imports)
+  src = pytd_utils.Print(
+      ast, fix_module_collisions=loader.options.fix_module_collisions)
+  ast = parser.parse_string(
+      src=src, name=module_name,
+      options=parser.PyiOptions.from_toplevel_options(loader.options))
   ast = ast.Visit(visitors.LookupBuiltins(loader.builtins, full_names=False))
   ast = ast.Visit(visitors.ExpandCompatibleBuiltins(loader.builtins))
   ast = ast.Visit(visitors.LookupLocalTypes())
