@@ -87,9 +87,12 @@ class FillInLocalPointers(Visitor):
 
   def _Lookup(self, node):
     """Look up a node by name."""
-    module, _, _ = node.name.rpartition(".")
-    if module:
-      modules_to_try = [("", module)]
+    if "." in node.name:
+      modules_to_try = []
+      module = node.name
+      while "." in module:
+        module, _, _ = module.rpartition(".")
+        modules_to_try.append(("", module))
     else:
       modules_to_try = [("", ""),
                         ("", "builtins"),
@@ -98,8 +101,13 @@ class FillInLocalPointers(Visitor):
     for prefix, module in modules_to_try:
       mod_ast = self._lookup_map.get(module)
       if mod_ast:
+        name = prefix + node.name
+        mod_prefix = f"{mod_ast.name}."
         try:
-          item = mod_ast.Lookup(prefix + node.name)
+          if name.startswith(mod_prefix) and mod_prefix != "builtins.":
+            item = pytd.LookupItemRecursive(mod_ast, name[len(mod_prefix):])
+          else:
+            item = mod_ast.Lookup(name)
         except KeyError:
           pass
         else:
@@ -210,25 +218,6 @@ class DropBuiltinPrefix(Visitor):
     return self.VisitClassType(node)
 
 
-# TODO(b/195453869): Get rid of this hack once we drop Python 2 support and all
-# typeshed references to __builtin__ are gone.
-def RenameBuiltinsPrefixInName(name):
-  module, _, basename = name.partition(".")
-  if module == "__builtin__":
-    return f"builtins.{basename}" if basename else "builtins"
-  return name
-
-
-class RenameBuiltinsPrefix(Visitor):
-  """Rename '__builtin__' to 'builtins' at import time."""
-
-  def VisitClassType(self, node):
-    return pytd.NamedType(RenameBuiltinsPrefixInName(node.name))
-
-  def VisitNamedType(self, node):
-    return self.VisitClassType(node)
-
-
 def LookupClasses(target, global_module=None, ignore_late_types=False):
   """Converts a PyTD object from one using NamedType to ClassType.
 
@@ -402,7 +391,6 @@ class LookupExternalTypes(RemoveTypeParametersFromGenericAny, _ToTypeVisitor):
       g = module.Lookup(module_name + ".__getattr__")
     except KeyError:
       return None
-    # TODO(b/159059370): Make parser.py actually enforce this:
     assert len(g.signatures) == 1
     return g.signatures[0].return_type
 
@@ -465,6 +453,8 @@ class LookupExternalTypes(RemoveTypeParametersFromGenericAny, _ToTypeVisitor):
         return t
       raise
     module_name = module.name
+    if module_name == self.name:  # dotted local reference
+      return t
     name = cls_prefix + name
     try:
       if name == "*":
@@ -632,6 +622,7 @@ class LookupLocalTypes(RemoveTypeParametersFromGenericAny, _ToTypeVisitor):
     super().__init__(allow_singletons)
     self._toplevel = toplevel
     self.local_names = set()
+    self.class_names = []
 
   def EnterTypeDeclUnit(self, unit):
     self.unit = unit
@@ -642,21 +633,43 @@ class LookupLocalTypes(RemoveTypeParametersFromGenericAny, _ToTypeVisitor):
   def _LookupItemRecursive(self, name):
     return pytd.LookupItemRecursive(self.unit, name)
 
+  def EnterClass(self, node):
+    self.class_names.append(node.name)
+
+  def LeaveClass(self, unused_node):
+    self.class_names.pop()
+
+  def _LookupScopedName(self, name):
+    """Look up a name in the chain of nested class scopes."""
+    scopes = [self.unit.name] + self.class_names
+    prefix = f"{self.unit.name}."
+    item = None
+    while item is None and scopes:
+      inner = scopes.pop()
+      lookup_name = f"{inner}.{name}"[len(prefix):]
+      try:
+        item = self._LookupItemRecursive(lookup_name)
+      except KeyError:
+        pass
+    return item
+
   def _LookupLocalName(self, node):
     assert "." not in node.name
     self.local_names.add(node.name)
-    try:
-      item = self.unit.Lookup(self.unit.name + "." + node.name)
-    except KeyError:
-      # Happens for infer calling load_pytd.resolve_ast() for the final pyi
+    item = self._LookupScopedName(node.name)
+    if item is None:
+      # Node names are not prefixed by the unit name when infer calls
+      # load_pytd.resolve_ast() for the final pyi.
       try:
         item = self.unit.Lookup(node.name)
-      except KeyError as e:
-        if (self.allow_singletons and node.name in pytd.SINGLETON_TYPES):
-          # Let the builtins resolver handle it
-          return node
-        raise SymbolLookupError("Couldn't find %s in %s" % (
-            node.name, self.unit.name)) from e
+      except KeyError:
+        pass
+    if item is None:
+      if (self.allow_singletons and node.name in pytd.SINGLETON_TYPES):
+        # Let the builtins resolver handle it
+        return node
+      msg = f"Couldn't find {node.name} in {self.unit.name}"
+      raise SymbolLookupError(msg)
     return item
 
   def _LookupLocalTypes(self, node):
@@ -666,8 +679,35 @@ class LookupLocalTypes(RemoveTypeParametersFromGenericAny, _ToTypeVisitor):
 
   def VisitNamedType(self, node):
     """Do lookup on a pytd.NamedType."""
-    module_name, dot, name = node.name.rpartition(".")
-    if not dot:  # simple reference to a member of the current module
+    # TODO(rechen): This method and FillInLocalPointers._Lookup do very similar
+    # things; is there any common code we can extract out?
+    if "." in node.name:
+      resolved_node = None
+      module_name, name = node.name, ""
+      while "." in module_name:
+        module_name, _, prefix = module_name.rpartition(".")
+        name = f"{prefix}.{name}" if name else prefix
+        if module_name == self.unit.name:
+          # Fully qualified reference to a member of the current module. May
+          # contain nested items that need to be recursively resolved.
+          try:
+            resolved_node = self.to_type(self._LookupItemRecursive(name))
+          except (KeyError, NotImplementedError):
+            if "." in name:
+              # This might be a dotted local reference without a module_name
+              # prefix, so we'll do another lookup attempt below.
+              pass
+            else:
+              raise
+          break
+      if resolved_node is None:
+        # Possibly a reference to a member of the current module that does not
+        # have a module_name prefix.
+        try:
+          resolved_node = self.to_type(self._LookupItemRecursive(node.name))
+        except KeyError:
+          resolved_node = node  # lookup failures are handled later
+    else:  # simple reference to a member of the current module
       item = self._LookupLocalName(node)
       if self._toplevel:
         # Check if the definition of this name refers back to itself.
@@ -685,17 +725,6 @@ class LookupLocalTypes(RemoveTypeParametersFromGenericAny, _ToTypeVisitor):
         resolved_node = self.to_type(item)
       except NotImplementedError as e:
         raise SymbolLookupError("%s is not a type" % item) from e
-    elif module_name == self.unit.name:
-      # Fully qualified reference to a member of the current module. May contain
-      # nested items that need to be recursively resolved.
-      resolved_node = self.to_type(self._LookupItemRecursive(name))
-    else:
-      # Possibly a reference to a member of the current module that does not
-      # have a module_name prefix.
-      try:
-        resolved_node = self.to_type(self._LookupItemRecursive(node.name))
-      except KeyError:
-        resolved_node = node  # lookup failures are handled later
     if isinstance(resolved_node, (pytd.Constant, pytd.Function)):
       visitor = LookupLocalTypes()
       visitor.unit = self.unit
@@ -804,7 +833,7 @@ class AdjustSelf(Visitor):
     return node
 
   def VisitParameter(self, p):
-    """Adjust all parameters called "self" to have their parent class type.
+    """Adjust all parameters called "self" to have their base class type.
 
     But do this only if their original type is unoccupied ("object" or,
     if configured, "Any").
@@ -1373,7 +1402,7 @@ class AdjustTypeParameters(Visitor):
     for x in params:
       if x.name in seen:
         raise ContainerError(
-            "Duplicate type parameter %s in typing.Generic parent of class %s" %
+            "Duplicate type parameter %s in typing.Generic base of class %s" %
             (x.name, class_name))
       seen.add(x.name)
 
@@ -1382,11 +1411,11 @@ class AdjustTypeParameters(Visitor):
     templates = []
     generic_template = None
 
-    for parent in node.parents:
-      if isinstance(parent, pytd.GenericType):
+    for base in node.bases:
+      if isinstance(base, pytd.GenericType):
         params = sum((self._GetTemplateItems(param)
-                      for param in parent.parameters), [])
-        if parent.name in ["typing.Generic", "Generic"]:
+                      for param in base.parameters), [])
+        if base.name in ["typing.Generic", "Generic"]:
           # TODO(mdemello): Do we need "Generic" in here or is it guaranteed
           # to be replaced by typing.Generic by the time this visitor is called?
           self._CheckDuplicateNames(params, node.name)
@@ -1430,7 +1459,7 @@ class AdjustTypeParameters(Visitor):
     self.class_template.pop()
 
   def VisitClass(self, node):
-    """Builds a template for the class from its GenericType parents."""
+    """Builds a template for the class from its GenericType bases."""
     # The template items will not have been properly scoped because they were
     # stored outside of the ast and not visited while processing the class
     # subtree.  They now need to be scoped similar to VisitTypeParameter,
@@ -1552,7 +1581,7 @@ class VerifyContainers(Visitor):
   """Visitor for verifying containers.
 
   Every container (except typing.Generic) must inherit from typing.Generic and
-  have an explicitly parameterized parent that is also a container. The
+  have an explicitly parameterized base that is also a container. The
   parameters on typing.Generic must all be TypeVar instances. A container must
   have at most as many parameters as specified in its template.
 
@@ -1610,7 +1639,7 @@ class VerifyContainers(Visitor):
     """
     mapping = collections.defaultdict(list)
     seen_bases = set()
-    bases = list(reversed(node.parents))
+    bases = list(reversed(node.bases))
     while bases:
       base = bases.pop()
       if base in seen_bases:
@@ -1619,9 +1648,9 @@ class VerifyContainers(Visitor):
       if (isinstance(base, pytd.GenericType) and
           isinstance(base.base_type, pytd.ClassType)):
         mapping[base.base_type].append(base)
-        bases.extend(reversed(base.base_type.cls.parents))
+        bases.extend(reversed(base.base_type.cls.bases))
       elif isinstance(base, pytd.ClassType):
-        bases.extend(reversed(base.cls.parents))
+        bases.extend(reversed(base.cls.bases))
     return mapping
 
   def _UpdateParamToValuesMapping(self, mapping, param, value):
