@@ -2,6 +2,7 @@
 
 import collections
 import re
+import sys
 from typing import Optional
 
 import attr
@@ -24,7 +25,12 @@ from pytype.tools.xref import callgraph
 from pytype.tools.xref import utils as xref_utils
 from pytype.tools.xref import node_utils
 
-from typed_ast import ast3
+# pylint: disable=g-import-not-at-top
+if sys.version_info >= (3, 8):
+  import ast as ast3
+else:
+  from typed_ast import ast3
+# pylint: enable=g-import-not-at-top
 
 
 # A mapping of offsets between a node's start position and the symbol being
@@ -901,7 +907,7 @@ class Indexer:
     self.loader = loader
     self.pytd_module = pytd_module
     self.resolved_modules = loader.get_resolved_modules()
-    self.imports = xref_utils.process_imports_map(loader.imports_map)
+    self.imports = xref_utils.process_imports_map(loader.options.imports_map)
     self.module_name = module_name
     self.traces = src.traces
     self.defs = None
@@ -1251,8 +1257,10 @@ def process_file(options, source_text=None, generate_callgraphs=False,
           loader=loader,
           ctx=ctx)
 
+  # pylint: disable=unexpected-keyword-arg
   ast_root_node = ast3.parse(src, options.input,
                              feature_version=options.python_version[1])
+  # pylint: enable=unexpected-keyword-arg
 
   # TODO(mdemello): Get from args
   module_name = "module"
