@@ -9,7 +9,6 @@ import subprocess
 import sys
 import threading
 import traceback
-import types
 from typing import List, Tuple
 import weakref
 
@@ -56,32 +55,8 @@ def format_version(python_version):
 
 
 def version_from_string(version_string):
-  """Parse a version string like "3" or "3.7" into a tuple."""
-  try:
-    version_int = int(version_string)
-  except ValueError:
-    return tuple(map(int, version_string.split(".")))
-  return full_version_from_major(version_int)
-
-
-# TODO(b/195453869): There's no longer any reason to accept just a major
-# version, since all supported versions are Python 3.x.
-def full_version_from_major(major_version):
-  """Get a (major, minor) Python version tuple from a major version int."""
-  if major_version == sys.version_info.major:
-    return sys.version_info[:2]
-  else:
-    raise UsageError(
-        "Cannot infer Python minor version for major version %d. "
-        "Specify the version as <major>.<minor>." % major_version)
-
-
-def normalize_version(version):
-  """Gets a version tuple from either a major version int or a version tuple."""
-  if isinstance(version, int):
-    return full_version_from_major(version)
-  else:
-    return version
+  """Parse a version string like "3.7" into a tuple."""
+  return tuple(map(int, version_string.split(".")))
 
 
 def validate_version(python_version):
@@ -252,75 +227,6 @@ def _arg_names(f):
   return f.__code__.co_varnames[:f.__code__.co_argcount]
 
 
-class memoize:  # pylint: disable=invalid-name
-  """A memoizing decorator that supports expressions as keys.
-
-  Use it like this:
-    @memoize
-    def f(x):
-      ...
-  or
-    @memoize("(id(x), y)")
-    def f(x, y, z):
-      ...
-  .
-  Careful with methods. If you have code like
-    @memoize("x")
-    def f(self, x):
-      ...
-  then memoized values will be shared across instances.
-
-  This decorator contains some speed optimizations that make it not thread-safe.
-  """
-
-  def __new__(cls, key_or_function):
-    if isinstance(key_or_function, types.FunctionType):
-      f = key_or_function
-      key = "(" + ", ".join(_arg_names(f)) + ")"
-      return memoize(key)(f)
-    else:
-      key = key_or_function
-      return object.__new__(cls)
-
-  def __init__(self, key):
-    self.key = key
-
-  def __call__(self, f):
-    key_program = compile(self.key, filename=__name__, mode="eval")
-    argnames = _arg_names(f)
-    memoized = {}
-    no_result = object()
-    if f.__defaults__:
-      defaults = dict(zip(argnames[-len(f.__defaults__):], f.__defaults__))
-    else:
-      defaults = {}
-    pos_and_arg_tuples = list(zip(range(f.__code__.co_argcount), argnames))
-    shared_dict = {}
-    # TODO(b/159037011): Use functools.wraps or functools.update_wrapper to
-    # preserve the metadata of the original function.
-    def call(*posargs, **kwargs):
-      """Call a memoized function."""
-      if kwargs or defaults:
-        # Slower version; for default arguments, we need two dictionaries.
-        args = defaults.copy()
-        args.update(dict(zip(argnames, posargs)))
-        args.update(kwargs)
-        key = eval(key_program, args)  # pylint: disable=eval-used
-      else:
-        # Faster version, if we have no default args.
-        for pos, arg in pos_and_arg_tuples:
-          # We know we write *all* the values, so we can re-use the dictionary.
-          shared_dict[arg] = posargs[pos]
-        key = eval(key_program, shared_dict)  # pylint: disable=eval-used
-      result = memoized.get(key, no_result)
-      if result is no_result:
-        # Call the actual function.
-        result = f(*posargs, **kwargs)
-        memoized[key] = result
-      return result
-    return call
-
-
 def invert_dict(d):
   """Invert a dictionary.
 
