@@ -68,6 +68,7 @@ class BytecodeTest(test_base.BaseTest, test_utils.MakeCodeMixin):
     self.ctx = context.Context(self.errorlog, self.options, self.loader)
     self.ctx.vm = TraceVM(self.ctx)
 
+  @test_utils.skipUnlessPy((3, 7), reason="Only testing one version.")
   def test_simple(self):
     # Disassembled from:
     # | return None
@@ -80,6 +81,7 @@ class BytecodeTest(test_base.BaseTest, test_utils.MakeCodeMixin):
     ctx.vm = vm.VirtualMachine(ctx)
     ctx.vm.run_bytecode(ctx.program.NewCFGNode(), code)
 
+  @test_utils.skipUnlessPy((3, 7), reason="Only testing one version.")
   def test_diamond(self):
     # Disassembled from:
     # | if []:
@@ -155,6 +157,7 @@ class BytecodeTest(test_base.BaseTest, test_utils.MakeCodeMixin):
       _OPMAP["RETURN_VALUE"], 0,
   ])
 
+  @test_utils.skipUnlessPy((3, 7), reason="Only testing one version.")
   def test_each_instruction_once_loops(self):
     code_nested_loop = pyc.compile_src(
         src=self.src_nested_loop,
@@ -185,6 +188,7 @@ class BytecodeTest(test_base.BaseTest, test_utils.MakeCodeMixin):
       _OPMAP["RETURN_VALUE"], 0,
   ])
 
+  @test_utils.skipUnlessPy((3, 7), reason="Only testing one version.")
   def test_each_instruction_once_dead_code(self):
     code_deadcode = pyc.compile_src(
         src=self.src_deadcode,
@@ -259,7 +263,8 @@ class AnnotationsTest(test_base.BaseTest, test_utils.MakeCodeMixin):
         })
 
 
-class DirectorLineNumbersTest(test_base.BaseTest, test_utils.MakeCodeMixin):
+class _DirectorLineNumbersTestCase(
+    test_base.BaseTest, test_utils.MakeCodeMixin):
 
   def setUp(self):
     super().setUp()
@@ -267,8 +272,12 @@ class DirectorLineNumbersTest(test_base.BaseTest, test_utils.MakeCodeMixin):
     self.ctx = context.Context(self.errorlog, self.options, self.loader)
 
   def run_program(self, src):
+    self.num_lines = len(src.rstrip().splitlines())
     return self.ctx.vm.run_program(textwrap.dedent(src), "", maximum_depth=10)
 
+
+class DirectorLineNumbersTest(_DirectorLineNumbersTestCase):
+
   def test_type_comment_on_multiline_value(self):
     self.run_program("""
       v = [
@@ -371,5 +380,137 @@ class DirectorLineNumbersTest(test_base.BaseTest, test_utils.MakeCodeMixin):
     self.assertEqual(self.ctx.vm._director.decorators, {5, 8})
 
 
+class FunctionCallDisableTest(_DirectorLineNumbersTestCase):
+
+  def assertDisables(self, *disable_lines, error_class="wrong-arg-types"):
+    disables = self.ctx.vm._director._disables[error_class]
+    for i in range(self.num_lines):
+      lineno = i+1
+      if lineno in disable_lines:
+        self.assertIn(lineno, disables)
+      else:
+        self.assertNotIn(lineno, disables)
+
+  def test_basic(self):
+    self.run_program("""
+      toplevel(
+          a, b, c, d)  # pytype: disable=wrong-arg-types
+    """)
+    if self.python_version >= (3, 8):
+      self.assertDisables(2)
+    else:
+      self.assertDisables(3)
+
+  def test_nested(self):
+    self.run_program("""
+      toplevel(
+          nested())  # pytype: disable=wrong-arg-types
+    """)
+    if self.python_version >= (3, 8):
+      self.assertDisables(2, 3)
+    else:
+      self.assertDisables(3)
+
+  def test_multiple_nested(self):
+    self.run_program("""
+      toplevel(
+        nested1(),
+        nested2(),  # pytype: disable=wrong-arg-types
+        nested3())
+    """)
+    if self.python_version >= (3, 8):
+      self.assertDisables(2, 3, 4)
+    else:
+      self.assertDisables(4)
+
+  def test_multiple_toplevel(self):
+    self.run_program("""
+      toplevel1()
+      toplevel2()  # pytype: disable=wrong-arg-types
+      toplevel3()
+    """)
+    self.assertDisables(3)
+
+  def test_deeply_nested(self):
+    self.run_program("""
+      toplevel(
+        nested1(),
+        nested2(
+          deeply_nested1(),  # pytype: disable=wrong-arg-types
+          deeply_nested2()),
+        nested3())
+    """)
+    if self.python_version >= (3, 8):
+      self.assertDisables(2, 3, 4, 5)
+    else:
+      self.assertDisables(5)
+
+  def test_trailing_parenthesis(self):
+    self.run_program("""
+      toplevel(
+          a, b, c, d,
+      )  # pytype: disable=wrong-arg-types
+    """)
+    if self.python_version >= (3, 8):
+      self.assertDisables(2)
+    else:
+      self.assertDisables(3)
+
+  def test_multiple_bytecode_blocks(self):
+    self.run_program("""
+      def f():
+        call(a, b, c, d)
+      def g():
+        call(a, b, c, d)  # pytype: disable=wrong-arg-types
+    """)
+    self.assertDisables(5)
+
+  def test_compare(self):
+    self.run_program("""
+      import datetime
+      def f(right: datetime.date):
+        left = datetime.datetime(1, 1, 1, 1)
+        return left < right  # pytype: disable=wrong-arg-types
+    """)
+    self.assertDisables(5)
+
+  def test_iterate(self):
+    self.run_program("""
+      class Foo:
+        def __iter__(self, too, many, args):
+          pass
+      foo = Foo()
+      for x in foo:  # pytype: disable=missing-parameter
+        print(x)
+    """)
+    self.assertDisables(6, error_class="missing-parameter")
+
+  def test_subscript(self):
+    self.run_program("""
+      class Foo:
+        def __getitem__(self, too, many, args):
+          pass
+      x = Foo()
+      x['X']  # pytype: disable=missing-parameter
+    """)
+    self.assertDisables(6, error_class="missing-parameter")
+
+  def test_attrs(self):
+    self.run_program("""
+      import attr
+      def converter(x):
+        return []
+      @attr.s
+      class Foo:
+        x = attr.ib(
+          converter=converter, factory=list, type=dict[str, str]
+        )  # pytype: disable=annotation-type-mismatch
+    """)
+    if self.python_version >= (3, 8):
+      self.assertDisables(7, error_class="annotation-type-mismatch")
+    else:
+      self.assertDisables(8, error_class="annotation-type-mismatch")
+
+
 if __name__ == "__main__":
   test_base.main()
